/***************************************************************************
                             buddylistwindow.cpp
                             -------------------
    begin                : Sun Feb 13 2000
    copyright            : (C) 2000 by Benjamin Meyer
    email                : ben@meyerhome.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "buddylistwindow.h"
#include "application.h"
#include "browser.h"
#include "info.h"
#include "chat.h"
#include <qdict.h>
#include <qfiledialog.h>
#include <qmessagebox.h>
#include <qpixmap.h>
#include <qbitmap.h>
#include <qlineedit.h>
#include <namedialog.h>
#include <qheader.h>
#include <qpushbutton.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qpe/resource.h>

#define DEFAULT_AWAYWIDGET_HEIGHT 60

//These delays are in seconds!!
#define POST_BUDDY_SIGNOFF_DELAY 10
#define POST_BUDDY_SIGNON_DELAY 10
#define POST_AWAY_MESSAGE_SENT_DELAY 300
#define POST_GET_AWAY_MESSAGE 3

#define TWINDOW_AIM_SANITY_CHECK if(connection->getStatus() == TAIM_OFFLINE) {\
                                   QMessageBox::critical(0, QString(APP_NAME), QString("You are not online."));\
                                   return; }

/***************************************************************************
 * Constructor
 ***************************************************************************/
BuddyListWindow::BuddyListWindow(QWidget *parent, const char *name, TAim *i_connection ) : QWidget(parent,name){
  viewAll = true;
  autoResizing = false;
  awayRightNow = false;
  muteSound = false;

  i_aimSettings = setup::settings();
  connection = i_connection;
  loadingTheList = false;

  connect(connection, SIGNAL(userInfoReceived(QString)), this, SLOT(slotBuddyInfoRecieved(QString)));
  connect(connection, SIGNAL(IMReceived(QString, QString, bool)), this, SLOT(receivedIM(QString, QString, bool)));
  connect(connection, SIGNAL(buddyChanged(int)), this, SLOT(buddyChanged(int)));
  connect(connection, SIGNAL(userSignedOn(QString)), this, SLOT(buddySignedOn(QString)));

  // Create the menu
  _FileMenu = new QPopupMenu(this, "FileMenu");
  _ToolsMenu = new QPopupMenu(this, "ToolsMenu");
  _AwayMenu = new QPopupMenu(this, "AwayMenu");
  _ViewMenu = new QPopupMenu(this, "ViewMenu");
  _BuddyMenu = new QPopupMenu(this, "Buddy");
  _FolderMenu = new QPopupMenu(this, "Folder");
  _HelpMenu = new QPopupMenu(this, "HelpMenu");
  _importMenu = new QPopupMenu(this, "ImportMenu");
  
  // The menu bar for the menus
  _menuBar= new QMenuBar(this, "NoName");
  initWindow();
  connect(_AwayMenu, SIGNAL(activated(int)), this, SLOT(away_GoAway(int)));
  connect(treeList->buddyEdit, SIGNAL(returnPressed()), this, SLOT(changeBuddyName()));
  connect(treeList, SIGNAL(removeBuddy(QString)), this, SLOT(slotRemoveBuddy(QString)));
  connect(treeList, SIGNAL(removeFolder(QString)), this, SLOT(slotRemoveFolder(QString)));

  messageWindows.setAutoDelete(false);
  _TreeMenu = new QPopupMenu(this, "NoName");
}

/******************************************************************************
 * Init anything in the main window that would change. 
 ******************************************************************************/
void BuddyListWindow::initOptions(){
  // No Icons option
  if (i_aimSettings->main_noIcons == true){
    if (chatButton->isHidden() == false){
      chatButton->hide();
      awayButton->hide();
      infoButton->hide();
    }
  }
  else{
    if (chatButton->isHidden() == true){
      chatButton->show();
      awayButton->show();
      infoButton->show();
    }
  }
  
  // Browser option
  browser->otherBrowser = i_aimSettings->main_browser;

  // Idle format option
  if (i_aimSettings->main_idleLayout == 1)
    connection->idleStringWColon = true;
  else
    connection->idleStringWColon = false;
  
  // Reload list for any other changes.
  loadList();
}

/***************************************************************************
 * init for the main objects, button, tree
 ***************************************************************************/
void BuddyListWindow::initWindow(){
  treeList= new KinkattaListView(this,"NoName");
  treeList->setGeometry(5,100,130,210);

  treeList->addColumn("Screen Name", -1);
  //treeList->setBackgroundPixmap(QPixmap("/home/ben/background.gif"));
  connect( treeList, SIGNAL(moveItem(QListViewItem *, QListViewItem *)), this, SLOT(moveBuddy(QListViewItem *, QListViewItem *)));
  connect( treeList, SIGNAL(expanded (QListViewItem *)), this, SLOT(folderExpanded (QListViewItem *)));
  connect( treeList, SIGNAL(collapsed(QListViewItem *)), this, SLOT(folderCollapsed(QListViewItem *)));

  connect( treeList, SIGNAL(doubleClicked(QListViewItem *)), this, SLOT(TreeSelected(QListViewItem *)) );
  connect( treeList, SIGNAL(returnPressed(QListViewItem *)), this, SLOT(TreeSelected(QListViewItem *)) );
  connect( treeList, SIGNAL(rightButtonClicked(QListViewItem *, const QPoint &, int)), this, SLOT(rightClickInTree(QListViewItem *, const QPoint &, int)) );

  chatButton= new QPushButton(this,"NoName");
  chatButton->setGeometry(5,315,40,40);
  chatButton->setPixmap(Icon("chat_big"));
  QToolTip::add(chatButton,"Chat");

  infoButton= new QPushButton(this,"NoName");
  infoButton->setGeometry(95,315,40,40);
  infoButton->setPixmap(Icon("info_big"));
  QToolTip::add(infoButton,"Get Buddy Info");

  awayButton= new QPushButton(this,"NoName");
  awayButton->setGeometry(50,315,40,40);
  awayButton->setPixmap(Icon("away_big"));
  QToolTip::add(awayButton,"Away");
	
  // The Three Button Connections.
  connect(chatButton, SIGNAL(clicked()), this, SLOT(chatWindow_ChatWithBuddy()));
  connect(awayButton, SIGNAL(pressed()), this, SLOT(away_ShowAwayList()));
  connect(infoButton, SIGNAL(clicked()), this, SLOT(slotGetBuddyInfo()));

  awayWidget = new AwayMessage(this, "AwayDialogBox");
  connect(awayWidget, SIGNAL(backClicked()), this, SLOT(away_BackForReal()));	
  connect(awayWidget, SIGNAL(update(QString)), this, SLOT(away_Update(QString)));
  awayWidget->hide();
}

/***************************************************************************
 * Builds the menu bars
 ***************************************************************************/
void BuddyListWindow::initMenuBar(){
  id_buddyList = _FileMenu->insertItem(Icon("icon_aim"), "&Buddy", _BuddyMenu);
    _BuddyMenu->clear();
    _BuddyMenu->insertItem(Icon("info"), "&Get Buddy Info", this, SLOT(slotGetBuddyInfo()));
    _BuddyMenu->insertItem(Icon("remove"), "&Remove Buddy...", this, SLOT(slotRemoveBuddy()));
    _BuddyMenu->insertItem(Resource::loadPixmap("edit"), "&Edit Name", this, SLOT(slotEditBuddy()));
    _BuddyMenu->insertItem(Resource::loadPixmap("edit"), "Edit &Alias", this, SLOT(slotEditAlias()));

  _FileMenu->insertItem(Icon("folder"), "&Folder", _FolderMenu);
    _FolderMenu->clear();
    _FolderMenu->insertItem(Icon("add"), "&Add Buddy...", this, SLOT(slotAddBuddy()));
    _FolderMenu->insertSeparator();
    _FolderMenu->insertItem(Icon("folder_create"), "Add &Folder", this, SLOT(slotAddFolder()));
    id_folderRemove = _FolderMenu->insertItem(Resource::loadPixmap("trash"), "&Remove Folder...", this, SLOT(slotRemoveFolder()));
    _FolderMenu->insertSeparator();
    id_folderEdit = _FolderMenu->insertItem(Resource::loadPixmap("edit"), "&Edit Name", this, SLOT(slotEditBuddy()));

  /*						
  _FileMenu->insertItem(Icon("import"), "&Import/Export", _importMenu);
    _importMenu->clear();
    _importMenu->insertItem(Icon("icon_gaim"), "&Gaim...", this, SLOT(slotImportFromGaim()));
    _importMenu->insertItem(Icon("icon_aim"), "&Tik...", this, SLOT(slotImportFromTik()));
    _importMenu->insertItem(Icon("kinkatta_small"), "K&inkatta...", this,SLOT(slotImportFromKinkatta()));
    _importMenu->insertSeparator();
    _importMenu->insertItem(Icon("import"), "&Export...", this, SLOT(slotExportBuddyList()), CTRL+Key_E);
  */
  _FileMenu->insertSeparator();
  _FileMenu->insertItem(Icon("chat"), "&New IM...", this, SLOT(chatWindow_ChatWithBuddy()), CTRL+Key_N);

  _FileMenu->insertSeparator();
  _FileMenu->insertItem(Resource::loadPixmap("SettingsIcon"),"&Options", this, SIGNAL(showOptions()), CTRL+Key_O);
  _FileMenu->insertSeparator();
  _FileMenu->insertItem(Icon("buddy_offline"), "&Log Off", this, SIGNAL(logOff()), CTRL+Key_L);
  _FileMenu->insertItem(Resource::loadPixmap("quit_icon"),"&Quit", this, SIGNAL(quit()), CTRL+Key_Q);
  connect(_FileMenu, SIGNAL(aboutToShow()), this, SLOT(aboutToShowFileMenu()));
  
  _ToolsMenu->insertItem(Icon("buddy_away"), "&Away", _AwayMenu);
    initAwayList();
  _ToolsMenu->insertItem(Icon("view"), "&View", _ViewMenu);
    initViewAllSetting();
		 	 
  _HelpMenu->insertItem(Icon("kinkatta_small"), "&About...", this, SLOT(MenuSlotAbout()));
  _menuBar->insertItem("&Program", _FileMenu);
  _menuBar->insertItem("&Tools", _ToolsMenu);
  _menuBar->insertItem("&Help", _HelpMenu);
  _menuBar->insertItem("|",1000);
  _menuBar->setItemEnabled(1000,false);
  _menuBar->insertItem("&Windows", windows);
}

/**
 *
 */
void BuddyListWindow::fillUpWindows(){
  windows->clear();
  // Insert windows
  QDictIterator<Chat> it( messageWindows );
  Chat *cw = NULL;
  while( it.current() ){
    cw = messageWindows[ it.currentKey() ];
    if(cw)
      windows->insertItem(Icon("chat"), cw->name, cw, SLOT(raise()));
    ++it;
  }

  // Insert standard
  if(messageWindows.count() > 0)
    windows->insertSeparator();
  windows->insertItem(Icon("kinkatta_small"), "&Main Window", parent(), SLOT(raise()));
  windows->insertItem(Resource::loadPixmap("SettingsIcon"), "&Options", this, SIGNAL(showOptions()));
}

/***************************************************************************
 * A buddy has been moved from one folder to another.  Move it. :)
 ***************************************************************************/
void BuddyListWindow::moveBuddy(QListViewItem* buddy, QListViewItem* folder){
  if (buddy != NULL && folder != NULL){
    TBuddyList list = connection->buddyList();
    int folderNumber = list.getNumGroup(((BuddyListWindowItem*)folder)->name);
    list.setGroup(list.getNum(((BuddyListWindowItem*)buddy)->name),folderNumber);
    connection->setBuddyList(list);
    loadList();
  }
}

/***************************************************************************
 * Loads the ViewAll settings.
 ***************************************************************************/
void BuddyListWindow::initViewAllSetting(){
  _ViewMenu->clear();
  _ViewMenu->setCheckable(true);
  if (viewAll == true){
    viewAll = false;
    _ViewMenu->insertItem("&All Buddies", this, SLOT(ChangeViewAllSetting()),0,1);
  }
  else{
    viewAll = true;
    _ViewMenu->insertItem("&On-Line Buddies", this, SLOT(ChangeViewAllSetting()),0,1);
  }
  _ViewMenu->insertItem("&LastOn", this, SLOT(viewLastOn()),0,2);
  _ViewMenu->insertItem("&Idle", this, SLOT(viewIdle()),0,3);
  _ViewMenu->setItemChecked(2,i_aimSettings->main_showLastOn);
  _ViewMenu->setItemChecked(3,i_aimSettings->main_showIdle);
}

/***************************************************************************
 * Changes the ViewAll setting and updates the list
 ***************************************************************************/
void BuddyListWindow::ChangeViewAllSetting(){
  initViewAllSetting();
  loadList();
}

/***************************************************************************
 * Turn on/off the LastOn colume
 ***************************************************************************/
void BuddyListWindow::viewLastOn(){
  if (i_aimSettings->main_showLastOn == true)
    i_aimSettings->main_showLastOn = false;
  else
    i_aimSettings->main_showLastOn = true;
  _ViewMenu->setItemChecked(2,i_aimSettings->main_showLastOn);
  loadList();
}

/***************************************************************************
 * Turn on/off the idle colume
 ***************************************************************************/
void BuddyListWindow::viewIdle(){
  if (i_aimSettings->main_showIdle == true)
    i_aimSettings->main_showIdle = false;
  else
    i_aimSettings->main_showIdle = true;
  _ViewMenu->setItemChecked(3,i_aimSettings->main_showIdle);
  loadList();
}

/***************************************************************************
 * The function will parse through all the names and put them up on the Tree
 ***************************************************************************/
void BuddyListWindow::loadList() {
  //if ( loadingTheList == true)
  // return;
	
  loadingTheList = true;
  treeList->clear();

  // remove all columns
  while((treeList->header())->count() > 1)
    treeList->removeColumn(1);

  if(connection->getStatus() == TAIM_OFFLINE) return;
  // we're online, so get going...
	
  // add columns as necessary, and headers
  if ((i_aimSettings->main_showIdle) == true)
    treeList->addColumn("Idle", -1);
  if ((i_aimSettings->main_showLastOn) == true)
    treeList->addColumn("Last On", -1);

	if (i_aimSettings->main_noHeader == true)
	  (treeList->header())->hide();
	else
		(treeList->header())->show();

  // get list to use...
  TBuddyList list = connection->buddyList();
  // Set the user list to be the same as

  // Add the groups
  QString groupName = "";
	for (int i=0; i < list.getCountGroup(); i++) {
		groupName = list.getNameGroup(i);
    BuddyListWindowItem *groupItem = new BuddyListWindowItem (treeList, groupName + "(0/0)");
    groupItem->name = groupName;
    groupItem->setPixmap(0, Icon("folder"));
    // Add all the buddies in this group
    for (int i2=0; i2 < list.getCount(); i2++) {
      TBuddy *tempBuddy = list.getByNum(i2);
      if (tempBuddy != NULL){
        // Go through the budies seeing if they are in the group.
        if (tempBuddy->group == i){
          if (tempBuddy->status == TAIM_ONLINE || viewAll == true){
						const TBuddyGroup *gc = list.getByNumG(tempBuddy->group);
					  if(gc) addTreeBuddy(tempBuddy, groupName, gc->size);
					}
				}
			}
    }
    if (groupItem->childCount() == 0){
			const TBuddyGroup *gc = list.getByNumG(i);
			QString size = QString("%1/%2").arg(groupItem->childCount()).arg(gc->size);
			groupItem->setText(0,groupItem->name + " (" + size + ")"); 
		}
		groupItem->setOpen(true);	
  }
	if (list.getCountGroup() == 0){
    list.addGroup(QString("Buddies"));
    // Update server & local list
    connection->setBuddyList(list);
    //connection->setServerBuddyList(list);
    viewAll = true;
    initViewAllSetting();  
    // Add the folder to the gui and bring up the edit box on it.
    BuddyListWindowItem *groupItem = new BuddyListWindowItem (treeList, "Buddies (0/0)");
    groupItem->name = "Buddies";
		groupItem->setPixmap(0, Icon("folder"));
    //treeList->setCurrentItem(groupItem);
  }
  if (treeList->currentItem() == 0)
    treeList->setCurrentItem(treeList->firstChild());    
  loadingTheList = false;
}

/***************************************************************************
 * This function will add a buddy to a folder.
 ***************************************************************************/
void BuddyListWindow::addTreeBuddy(TBuddy *buddy, QString folder, int folder_size){
  BuddyListWindowItem *parent = (BuddyListWindowItem*)(treeList->firstChild());
	// Find the buddy in the tree
  while( parent != 0 ){
    if( ((BuddyListWindowItem*)(parent))->name == folder ){
	    QString alias = buddy->alias;	
      QString buddyName = buddy->name;
	  	if (alias.length() == 0)
	    	alias = buddyName; 	
			BuddyListWindowItem *buddyItem = new BuddyListWindowItem(parent, QString(""));
			if (alias != buddyName)
			  buddyItem->setText(0, QString("@") + alias);
			else
			  buddyItem->setText(0, buddyName);
			buddyItem->name = buddyName;
			loadTreeBuddy(buddy);
  	 
			// Add number to group. 
			QString size = QString("%1/%2").arg(parent->childCount()).arg(folder_size);
			parent->setText(0,parent->name + " (" + size + ")"); 
			break;
		}
  	parent = (BuddyListWindowItem*)(parent->nextSibling());
  }
}

/***************************************************************************
 * This function will fill the tree from the current buddy
 ***************************************************************************/
void BuddyListWindow::loadTreeBuddy(TBuddy *buddyInfo){
  if(connection->getStatus() == TAIM_OFFLINE)
    return;

  BuddyListWindowItem *parent = (BuddyListWindowItem*)(treeList->firstChild());
  BuddyListWindowItem *item = 0;
  BuddyListWindowItem *item2 = 0;

  // Check if it is a folder.
  if(parent == 0)
    return;
  
  // Find the buddy in the tree
  while( parent != 0 ){
    item = (BuddyListWindowItem*)parent->firstChild();
    while( item != 0 ){
      if( ((BuddyListWindowItem*)(item))->name == buddyInfo->name ){
	      item2 = (BuddyListWindowItem*)item;
  	    break;
      }
      item = (BuddyListWindowItem*)(item->nextSibling());
    }
    parent = (BuddyListWindowItem*)(parent->nextSibling());
  }
  
  if (item2 == 0 ){
    // The buddy must have just signed on.
    if (buddyInfo->status != TAIM_ONLINE && viewAll == false)
		  return;

		TBuddyList list = connection->buddyList();
		const TBuddyGroup *gc = list.getByNumG(buddyInfo->group);
    if(gc) addTreeBuddy(buddyInfo, list.getNameGroup(buddyInfo->group), gc->size);
    
    if((i_aimSettings->SoundMode == 1) &&
      (i_aimSettings->sound_enable_login == true) && !muteSound){
     	beep();
    }
    return;
  }

  // We have a buddy
  if (i_aimSettings->main_showIdle == true){
    if (treeList->columnText(1) == "Idle")
 	    item2->setText(1,buddyInfo->idleTimeString);
    if (treeList->columnText(2) == "Idle")
	    item2->setText(2,buddyInfo->idleTimeString);
  }
  if (i_aimSettings->main_showLastOn == true){
    if (treeList->columnText(2) == "Last On")
      item2->setText(2,buddyInfo->lastOnString); 
    if (treeList->columnText(1) == "Last On")
      item2->setText(1,buddyInfo->lastOnString);
  }

	// Set the icon
	QPixmap icon;
	if (buddyInfo->status !=  TAIM_ONLINE){
    ((BuddyListWindowItem *)item2)->setDecoration(BuddyListWindowItem::Offline);
    icon = Icon("buddy_offline");
  }
  else{
    //if (buddyInfo->signedOn == 1){
    //  icon =  Icon("login_icon");
    //  ((BuddyListWindowItem *)item2)->setDecoration(BuddyListWindowItem::SignOn);
    //}
    //		else{
			switch(buddyInfo->userClass){
        case TOC_USER_ADMIN:
          icon = Icon("buddy_aol");
          break;
        case TOC_USER_AOL:
          icon = Icon("buddy_aol");
          break;
        case TOC_USER_UNCONFIRMED:
          icon = Icon("icon_aim");
          break;
        case TOC_USER_NORMAL:
          icon = Icon("icon_aim");
          break;
        case TOC_USER_AWAY:
          icon = Icon("buddy_away");
          break;
        default:
          icon = Icon("icon_aim");
          break;
      //}
      ((BuddyListWindowItem *)item2)->setDecoration(BuddyListWindowItem::Online);
    }
  }
  QString buddyName = tocNormalize(buddyInfo->name);
  item2->setPixmap(0, icon);
}

/***************************************************************************
 * This will close all the existing open chatwindows.
 ***************************************************************************/
void BuddyListWindow::closeExistingChatwindows(){
  QDictIterator<Chat> it( messageWindows );
  Chat *cw;

  while( it.current() ){
    cw = messageWindows[ it.currentKey() ];
    if(cw)
      delete cw;
    messageWindows.remove( it.currentKey() );
    ++it;
  }
}

/***************************************************************************
 * This will clean up all the connections to the post* slots, and the
 * QValueLists
 ***************************************************************************/
void BuddyListWindow::clearPostConnections(void){
	QValueList<QString *>::Iterator it;

	for(it = signedOff.begin(); it != signedOff.end(); ++it){
		delete (*it);
	}
	signedOff.clear();

	for(it = signedOn.begin(); it != signedOn.end(); ++it){
		delete (*it);
	}
	signedOn.clear();

	for(it = awayMessageSent.begin(); it != awayMessageSent.end(); ++it){
		delete (*it);
	}
	awayMessageSent.clear();

	//disconnect the signals. does this disconnect the singleShots too??
	disconnect( this, SLOT(postBuddySignoff()) );
	disconnect( this, SLOT(postBuddySignon()) );
}

/***************************************************************************
 * Removes a buddy from the tree.  Called a little bit after they sign off.
 ***************************************************************************/
void BuddyListWindow::postBuddySignoff( ){
  QString *bud = 0;
	if( signedOff.count() != 0)
	 bud = signedOff.first();

  if(!bud)
    return;

	// If the buddy is actually on then don't remove him.
	TBuddyList list = connection->buddyList();
  int num = list.getNum(*bud);
  if (num == -1)
    return;
  TBuddy *buddy = list.getByNum(num);
  // Only remove him if he is not online right now
	if (buddy->status != TAIM_ONLINE){
	  if(!viewAll){ //remove the item from view only if we're not in view-All mode
	    QListViewItemIterator it( treeList );
	    for ( ; it.current(); it++ ){
				QString cName = ((BuddyListWindowItem*)it.current())->name;
	      if( cName == *bud ){
	        if(it.current()->parent()){
						
						TBuddy *b = list.getByNum( list.getNum(cName));
						if (b != NULL){
						  const TBuddyGroup *gc = list.getByNumG(b->group);
						  // Remove a number to group. 
						  QString size = QString("%1/%2").arg( it.current()->parent()->childCount()-1 ) .arg(gc->size);
							it.current()->parent()->setText(0,((BuddyListWindowItem*)it.current()->parent())->name + " (" + size + ")"); 
						}
						else
							qDebug("Can't find buddy");

						it.current()->parent()->removeItem( it.current() );

	        }	
	        break;
	      }
	    }
  	}
	}
	
  if (signedOff.isEmpty() != true){
    signedOff.remove( signedOff.begin() );
  }

  //dont leak memory:
  if(bud){
    delete bud;
  }
}

/***************************************************************************
 * Changes the icon of a buddy from just signed on to something else.
 ***************************************************************************/
void BuddyListWindow::postBuddySignon( ){
  QString *bud = 0;
  if( signedOn.count() != 0)
    bud = signedOn.first();

  if(!bud)
    return;

	TBuddyList list = connection->buddyList();
  int num = list.getNum(*bud);
  if (num == -1)
    return;

  TBuddy *buddy = list.getByNum(num);
  if (buddy){
		buddy->signedOn = 0;
	  loadTreeBuddy(buddy);
		connection->setBuddyList(list);
	}
	if (signedOn.isEmpty() != true){
    signedOn.remove( signedOn.begin() );
  }
 	
	//dont leak memory:
  if(bud){
    delete bud;
  }
}

/***************************************************************************
 * When a folder is expanded give it the open icon
 ***************************************************************************/
void BuddyListWindow::folderExpanded ( QListViewItem* item ){
  item->setPixmap(0, Icon("folder"));
} 

/***************************************************************************
 * When a folder is closed give it the closed icon
 ***************************************************************************/
void BuddyListWindow::folderCollapsed ( QListViewItem* item ){
  item->setPixmap(0, Icon("folder_closed"));
}

/***************************************************************************
 * The server will update a buddy
 ***************************************************************************/
void BuddyListWindow::buddyChanged(int number){
  TBuddyList list = connection->buddyList();
  TBuddy *bud = list.getByNum(number);
  loadTreeBuddy(bud);	
  // If signed off tell chat window and play sound if needed
  if (bud->status !=  TAIM_ONLINE){
    	
  // Check to see if there is a sound configure on this buddy. 
  bool playedSound = false;
    
    if (playedSound == false) {
      if((i_aimSettings->SoundMode == 1) && (i_aimSettings->sound_enable_logout == true) && !muteSound){
      	beep();
      }
    }
    QString ChatWindowName = i_aimSettings->login_name + tocNormalize(bud->name);
    Chat *targetWindow = messageWindows[ ChatWindowName ];
    if(targetWindow)
      targetWindow->buddySignedOff();

    //add to the signedOff users list
    signedOff += new QString(bud->name);
    // Timer to remove item
    QTimer::singleShot( POST_BUDDY_SIGNOFF_DELAY *1000, this, SLOT(postBuddySignoff())); 
  }  
}

/***************************************************************************
 * Buddy Just signed on tell any chat window that need to know
 ***************************************************************************/
void BuddyListWindow::buddySignedOn(QString buddy){
  QString ChatWindowName = i_aimSettings->login_name + tocNormalize(buddy);
  Chat *targetWindow = messageWindows[ ChatWindowName ];
  if(targetWindow)
    targetWindow->buddySignedOn();
	
  TBuddyList list = connection->buddyList();
  TBuddy *bud = list.getByNum(list.getNum(buddy));
  if (bud){
    bud->signedOn = 1;
    loadTreeBuddy(bud);
    signedOn += new QString(buddy);
    //QTimer::singleShot( POST_BUDDY_SIGNON_DELAY *1000, this, SLOT(postBuddySignon()));
    connection->setBuddyList(list);
  }
} 

/***************************************************************************
 * Item in tree was selected/double clicked.  Open chat window
 ***************************************************************************/
void BuddyListWindow::DoubleClicked(){
  if (treeList->currentItem() == 0)
    treeList->setCurrentItem(treeList->firstChild());
  TreeSelected(treeList->selectedItem());  
}

/***************************************************************************
 * Item in tree was selected/double clicked.  Open chat window
 ***************************************************************************/
void BuddyListWindow::TreeSelected(QListViewItem* selected){
  // If somehow we got a NULL just return 
  if (selected == NULL)
    return;
  // If not a folder
  if (treeList->checkIfFolder(selected) == false){
    const QString buddyName = ((BuddyListWindowItem*)selected)->name;
    Chat *targetWindow = messageWindow(buddyName);
    if (targetWindow != 0)
      targetWindow->show();
  }
}

/***************************************************************************
 * Settings changed tell things to update
 ***************************************************************************/
void BuddyListWindow::settingsChanged(){
  initOptions();
  emit (updateChatSettings());
  emit (updateChatSoundSettings());
  initAwayList();
}

/***************************************************************************
 * Add a buddy to the list
 ***************************************************************************/
void BuddyListWindow::slotAddBuddy(){
  //Make sure a folder exists and is selected
  if (treeList->currentItem() == 0)
    treeList->setCurrentItem(treeList->firstChild());    
  if(!treeList->selectedItem())
  {
    QMessageBox::warning( 0, APP_NAME " - Add Buddy",
      "Please select a folder",
      "OK", /*"Quit",*/ 0,
      0, 1 );
    return;
  }
  NameDialog name(this, "name dialog", true);
  QString text = name.go();
  if ( !text.isEmpty() )
    slotAddBuddy(text);// user entered something and pressed ok
  else
    return	;// user entered nothing or pressed cancel
  emit (saveSettings());
}

/***************************************************************************
 * Add a buddy to the list (buddyName)
 ***************************************************************************/
void BuddyListWindow::slotAddBuddy(QString buddyName){
  TBuddyList list = connection->buddyList();
  int groupNumber = 0;
  BuddyListWindowItem *cItem = (BuddyListWindowItem*)(treeList->selectedItem());
  // If window is highlighted get group number.
  if ( cItem != NULL){
    if (treeList->checkIfFolder(cItem) == true){
      groupNumber = list.getNumGroup(cItem->name);
    }
    else{
      //get parent
      BuddyListWindowItem *parent = (BuddyListWindowItem*)(cItem->parent());
      groupNumber = list.getNumGroup(parent->name);
    }
  }	
      
  // Update server
  TBuddy *edited = new TBuddy();
  edited->name = buddyName;
  edited->group = groupNumber;
  edited->status = TAIM_OFFLINE;
  edited->userClass = 0;
  edited->signonTime = 0;
  edited->evil = 0;

  list.add(edited);
  connection->setBuddyList(list);
  loadList();
  delete edited;
  
  emit updateIfIsBuddy(buddyName, true);
  emit (saveSettings());
}

/***************************************************************************
 * Removes a buddy from the list
 * Gets currently selected buddy and calls slotRemoveBuddy(name)
 ***************************************************************************/
void BuddyListWindow::slotRemoveBuddy(){
  QString name = treeList->currentName();
  slotRemoveBuddy(name);
  emit (saveSettings());
}

/******************************************************************************
 * Remove the buddy from the current list
 * @Peram name - Name of the buddy that is to be removed
 *****************************************************************************/
void BuddyListWindow::slotRemoveBuddy(QString name){
  int answer = QMessageBox::warning(0, APP_NAME " - Remove Buddy",
    "Do you wish to remove this\nbuddy from your buddylist?",
    "Yes", "Cancel", 0,1);
  if (!answer){
    TBuddyList list = connection->buddyList();
    list.del(list.getNum(name));
    connection->setBuddyList(list);
    //connection->setServerBuddyList(list);
    loadList();
    emit updateIfIsBuddy(name, false);
  }
}

/***************************************************************************
 * Add a new folder to the list.
 ***************************************************************************/
void BuddyListWindow::slotAddFolder(){
  // Get current list
  TBuddyList list = connection->buddyList();
  list.addGroup(QString("NewFolder"));
  // Update server list
  connection->setBuddyList(list);
  //connection->setServerBuddyList(list);
  viewAll = true;
  initViewAllSetting();  
  // Add the folder to the gui and bring up the edit box on it.
  BuddyListWindowItem *groupItem = new BuddyListWindowItem (treeList, "NewFolder (0/0)");
  groupItem->setPixmap(0, Icon("folder"));
  groupItem->name = "NewFolder";
	treeList->setCurrentItem(groupItem);
  slotEditBuddy();
  treeList->setSorting(0,1);
  treeList->sort();
  emit (saveSettings());
}

/***************************************************************************
 * Remove the current selected folder
 ***************************************************************************/
void BuddyListWindow::slotRemoveFolder(){
	if( treeList->currentItem() )
		slotRemoveFolder( treeList->currentName() );
}

/***************************************************************************
 * Remove the passed folder
 ***************************************************************************/
void BuddyListWindow::slotRemoveFolder(QString group){
  int answer = QMessageBox::warning(this, APP_NAME " - Remove Folder",
    "Do you wish to remove this\nfolder from your buddylist?\nAll buddies in the folder\nwill be lost.",
    "Yes", "Cancel", 0,1);
  if (!answer){
    TBuddyList list = connection->buddyList();
    list.delGroup(list.getNumGroup( group ));
    connection->setBuddyList(list);
    //connection->setServerBuddyList(list);
    loadList();
  }  
  emit (saveSettings());
}

/***************************************************************************
 * Get information button clicked, get current highlighted name and get info
 ***************************************************************************/
void BuddyListWindow::slotGetBuddyInfo(){
  if (treeList->currentItem() == 0)
    treeList->setCurrentItem(treeList->firstChild());    
  const QString name = treeList->currentName();
  if (treeList->checkIfFolder(treeList->currentItem()) == false){
    slotGetBuddyInfo( name );
  }
  else{
    NameDialog name(this, "name dialog", true);
    QString text = name.go();
    if ( !text.isEmpty() )
      slotGetBuddyInfo( text );
  }
}

/***************************************************************************
 * Get Information on a user
 ***************************************************************************/
void BuddyListWindow::slotGetBuddyInfo(QString name){
  TWINDOW_AIM_SANITY_CHECK;
  if(awayRightNow == true)
    away_BackForReal();
  connection->requestUserInfo( name );
}

/***************************************************************************
 * Info has been recieved
 * @Peram string - The info that was recieved
 ***************************************************************************/
void BuddyListWindow::slotBuddyInfoRecieved(QString string){
  QString name = i_aimSettings->login_name;
  Info *infoBox = new Info(0, "infoBox");
  infoBox->user = name;
  infoBox->addToWindow(string);
  infoBox->setCaption(APP_NAME " - Information");
  infoBox->setIcon(Icon("kinkatta_small"));
  infoBox->browser = browser;
  infoBox->show();
}

/***************************************************************************
 * Bring up the edit box on the currently selected user.
 ***************************************************************************/
void BuddyListWindow::slotEditBuddy(){
  QString name = "";
  BuddyListWindowItem *cItem = treeList->currentItem();
  name = cItem->name;
  
  treeList->buddyEdit->setText("");
  treeList->buddyEdit->setText(name);

  QRect r = treeList->itemRect(cItem);
  int x = r.x()+35;
  if (treeList->checkIfFolder(cItem) == true)
    x = x - 19;
  int y = r.y();
  int width = treeList->columnWidth(0)-x;
  int height = r.height();

  if(width < 0) width=0;
  if(height < 0) height=0;
  treeList->buddyEdit->setGeometry(x,y,width,height);
  treeList->moveChild(treeList->buddyEdit,x,y);
  treeList->buddyEdit->setMinimumSize(width,height);
  treeList->buddyEdit->setMaximumSize(width,height);

  // Showing it will give it focus
  treeList->buddyEdit->focus = 0;
  treeList->buddyEdit->alias = false;
  treeList->buddyEdit->show();
  treeList->buddyEdit->setFocus();
  treeList->buddyEdit->setSelection(0,treeList->buddyEdit->maxLength());
}

/***************************************************************************
 * Bring up the edit box on the currently selected user.
 ***************************************************************************/
void BuddyListWindow::slotEditAlias(){
  QString name = "";
  BuddyListWindowItem *cItem = treeList->currentItem();
  if (treeList->checkIfFolder(cItem)){
		// Folders don't have aliases	
		return;
  }
  name = cItem->text(0);
  // Remove @
	if (name != cItem->name)
	  name = name.mid(1, name.length()-1);

  treeList->buddyEdit->setText("");
  treeList->buddyEdit->setText(name);

	QRect r = treeList->itemRect(cItem);
  int x = r.x()+35;
	int y = r.y();
	int width = treeList->columnWidth(0)-x;
	int height = r.height();
	
	treeList->buddyEdit->setGeometry(x,y,width,height);
  treeList->moveChild(treeList->buddyEdit,x,y);
	treeList->buddyEdit->setMinimumSize(width,height);
  treeList->buddyEdit->setMaximumSize(width,height);
		
  // Showing it will give it focus
  treeList->buddyEdit->focus = 0;
	treeList->buddyEdit->alias = true;
  treeList->buddyEdit->show();
  treeList->buddyEdit->setFocus();
  treeList->buddyEdit->setSelection(0,treeList->buddyEdit->maxLength());
}

/***************************************************************************
 * Once the edit window is gone update the name
 ***************************************************************************/
void BuddyListWindow::changeBuddyName(){
  BuddyListWindowItem *cItem = treeList->currentItem();
  QString oldName = "";
  QString realName = "";
	
  if ( cItem == NULL)
    return;
  
  realName = cItem->name;
  if (treeList->buddyEdit->alias == true){
    oldName = cItem->text(0);
    if (oldName != realName)
      oldName = oldName.mid(1, oldName.length()-1);
    }
  else
     oldName = cItem->name;

  QString newName = treeList->buddyEdit->text();

  // Update List
  TBuddyList list = connection->buddyList();
  if (treeList->checkIfFolder(cItem) == true){
    list.renameGroup( oldName, newName);
    list.renameGroup( oldName, newName );
    connection->setBuddyList(list);
    loadList();
  }
  else{
    TBuddy *edited = list.getByNum(list.getNum(realName));
    if (edited){
      if (treeList->buddyEdit->alias == true)
        edited->alias = newName;
      else
        edited->name = newName;	
        connection->setBuddyList(list);
	loadList();
    }
  }

  treeList->buddyEdit->hide();
  emit (saveSettings());
}

/***************************************************************************
 * The tree was clicked with the right mouse button, open  the menu. 
 ***************************************************************************/
void BuddyListWindow::rightClickInTree(QListViewItem *, const QPoint &pos, int){
  _TreeMenu->clear();
  if (treeList->currentItem() == 0)
    treeList->setCurrentItem(treeList->firstChild());
  if(treeList->currentItem() != NULL)	{
    QString buddyName = treeList->currentName();
    // Check for folder
    QListViewItem *selected = treeList->currentItem();
    if (treeList->checkIfFolder(selected) == true){
      _TreeMenu->insertItem(Icon("add"), "&Add Buddy...", this, SLOT(slotAddBuddy()));
      _TreeMenu->insertSeparator();
      _TreeMenu->insertItem(Icon("folder_create"), "Add &Folder", this, SLOT(slotAddFolder()));
      _TreeMenu->insertItem(Resource::loadPixmap("trash"), "&Remove Folder...", this, SLOT(slotRemoveFolder()));
      _TreeMenu->insertSeparator();
      _TreeMenu->insertItem(Resource::loadPixmap("edit"), "&Edit Folder", this, SLOT(slotEditBuddy()));
    }
    else {
      _TreeMenu->insertItem(Icon("chat"), "&New Instant Message", this, SLOT(DoubleClicked()));
      _TreeMenu->insertItem(Icon("info"), "&Get Buddy Info", this, SLOT(slotGetBuddyInfo()));
      _TreeMenu->insertItem(Icon("remove"), "&Remove Buddy...", this, SLOT(slotRemoveBuddy()));
      _TreeMenu->insertItem(Resource::loadPixmap("edit"), "&Edit Buddy", this, SLOT(slotEditBuddy()));
      _TreeMenu->insertItem(Resource::loadPixmap("edit"), "Edit &Alias", this, SLOT(slotEditAlias()));
    }
  }	
  _TreeMenu->insertSeparator();
  _TreeMenu->insertItem(Icon("view"), "&View", _ViewMenu);
  _TreeMenu->exec(pos);
}

/***************************************************************************
 * We're about to show the file menu, so set the appropriate items in the buddy
 * popup menu and the folder popup menu activated or inactivated.
 ***************************************************************************/
void BuddyListWindow::aboutToShowFileMenu(){
  QListViewItem *selected = treeList->selectedItem();
  if(!selected){
    _FolderMenu->setItemEnabled(id_folderEdit, FALSE);
    _FolderMenu->setItemEnabled(id_folderRemove, FALSE);
    _FileMenu->setItemEnabled(id_buddyList, FALSE);
    return;
  }
  
  if( treeList->checkIfFolder(selected) ){
    _FolderMenu->setItemEnabled(id_folderEdit, TRUE);
    _FolderMenu->setItemEnabled(id_folderRemove, TRUE);
    _FileMenu->setItemEnabled(id_buddyList, FALSE);
  } else { //not a folder
    _FolderMenu->setItemEnabled(id_folderEdit, FALSE);
    _FolderMenu->setItemEnabled(id_folderRemove, FALSE);
    _FileMenu->setItemEnabled(id_buddyList, TRUE);
  }
}

/***************************************************************************
 * About - Displays the about box
 ***************************************************************************/
void BuddyListWindow::MenuSlotAbout(){
  QMessageBox *about= new QMessageBox(0, "About " APP_NAME);
  about->setCaption("About " APP_NAME "...");
  about->setText(
  (QString)APP_NAME " 1.0 (" + i_aimSettings->ReleaseDate +")\n"
  "(c) 2001 Benjamin Meyer\n"
  "ben@meyerhome.net\n"
  "\n"
  "Contributions By:\n"
  "Benjamin Meyer ben@meyerhome.net\n"
  "Dimi Shahbaz dshahbaz@ucla.edu\n"
  "Jim Wilson jimw@kelcomaine.com\n"
  "Neil Stevens multivac@fcmail.com\n" 
  "\n"
  "For more information you can visit \n"
  "the " APP_NAME " webpage at\n"
  "\"http://kinkatta.sourceforge.net/\".\n"
  "\n"
  "This program is dual licensed under\n"
  "the GPL and LGPL license. Visit \n"
  "\"http://www.gnu.org\" for more\n"
  "information.");
  about->setIconPixmap(Icon("kinkatta"));
  about->setIcon(Icon("kinkatta_small"));
  about->show();
  about->showMaximized();
}

/***************************************************************************
 * Brings up the Menu when the right mouse is clicked.
 ***************************************************************************/
void BuddyListWindow::mouseReleaseEvent(QMouseEvent* event){
  if (event->button() == RightButton){
    _AwayMenu->hide();
  }
}

/***************************************************************************
 * When signed on the option to turn off the sound would call a 10 second 
 * timer that would in turn call this to turn it back on.
 ***************************************************************************/
void BuddyListWindow::endMutePeriod(){
  muteSound = FALSE;
}

/***************************************************************************
 * Resizes the object in the window, when the window is resized.
 ***************************************************************************/
void BuddyListWindow::resizeEvent ( QResizeEvent* ) {
  autoResizing = true;
  int mid = width()/2;
  
  if(awayRightNow == false){
    if (i_aimSettings->main_noIcons == true){
       treeList->setGeometry (0, _menuBar->height(), width(), height()-28);	
    }
    else{
      treeList->setGeometry (0, _menuBar->height(), width(), height()-75);
      chatButton->setGeometry (mid-65, height()-45, 40, 40);
      awayButton->setGeometry (mid-20, height()-45, 40, 40);
      infoButton->setGeometry (mid+25, height()-45, 40, 40);
    }
  }
  else { //awayRightNow == true
    if (i_aimSettings->main_noIcons == true){
      treeList->setGeometry (0, _menuBar->height(), width(), height()-28-DEFAULT_AWAYWIDGET_HEIGHT);	
      awayWidget->setGeometry(0, height()-DEFAULT_AWAYWIDGET_HEIGHT, treeList->width(), DEFAULT_AWAYWIDGET_HEIGHT);
    }
    else{
     treeList->setGeometry (0, _menuBar->height(), width(), (height()-75-DEFAULT_AWAYWIDGET_HEIGHT));
     chatButton->setGeometry (mid-65, height()-45-DEFAULT_AWAYWIDGET_HEIGHT, 40, 40);
     awayButton->setGeometry (mid-20, height()-45-DEFAULT_AWAYWIDGET_HEIGHT, 40, 40);
     infoButton->setGeometry (mid+25, height()-45-DEFAULT_AWAYWIDGET_HEIGHT, 40, 40);
     awayWidget->setGeometry (0, height()-DEFAULT_AWAYWIDGET_HEIGHT, width(), DEFAULT_AWAYWIDGET_HEIGHT);
    }
  }
  autoResizing = false;
}


/**
 *
 *
 * The away Functions.
 *
 *
 */

/***************************************************************************
 * 5 minutes after a away message is sent the send away message again is
 * turned back on. 
 ***************************************************************************/
void BuddyListWindow::postAwayMessageSent( ){
	QString *bud = 0;
	if( awayMessageSent.count() != 0)
  	bud = awayMessageSent.first();

  if(!bud)
    return;
  QString normedName = tocNormalize(*bud);
  QString ChatWindowName = (i_aimSettings->login_name) + normedName;
  Chat*targetWindow = messageWindows[ChatWindowName];
  if(targetWindow){
    targetWindow->sentAwayMessageToBuddy = false;
  }    

  if (awayMessageSent.isEmpty() != true){
		awayMessageSent.remove( awayMessageSent.begin() );
	}
	
	//dont leak memory:
  if(bud){
    delete bud;
  }
}

/***************************************************************************
 * Loads the Awaylist under the list.
 ***************************************************************************/
void BuddyListWindow::initAwayList(){
  _AwayMenu->clear();
  QMap <QString, QString>::Iterator it;
  int count = 1;
  for(it = i_aimSettings->mapAwayMessages->begin(); 
    it != i_aimSettings->mapAwayMessages->end(); ++it){
    _AwayMenu->insertItem(it.key(), count++);
  }
  _AwayMenu->insertSeparator();
  _AwayMenu->insertItem("&New Message...", this, SIGNAL(showAwayOptions()));
}

/***************************************************************************
 * The away button was clicked show the list there.
 ***************************************************************************/
void BuddyListWindow::away_ShowAwayList(){
  initAwayList();
  QPoint temp(parentWidget()->x()+awayButton->x()+20,parentWidget()->y()+awayButton->y()+50);
  _AwayMenu->exec(temp);
  awayButton->setDown(false);
}

/***************************************************************************
 * Find out what number name is in the list and call away(int)
 ***************************************************************************/
void BuddyListWindow::away_makeAwayNow(QString name){
  int count = 1;
  int choice = -1;
	QMap <QString, QString>::Iterator it;
	for(it = i_aimSettings->mapAwayMessages->begin(); 
			it != i_aimSettings->mapAwayMessages->end(); ++it){
		if(it.key() == name){
			choice = count;
		}
		count++;
	}
  if (choice != -1){
    away_GoAway(choice);
	}
}

/***************************************************************************
 * Set the away message to text
 ***************************************************************************/
void BuddyListWindow::away_Update(QString text){
  connection->setAwayMessage(text);
  awayMessage = text;
}

/***************************************************************************
 * Set Kinkatta to away mode.
 ***************************************************************************/
void BuddyListWindow::away_GoAway(int num){
	QMap <QString, QString>::Iterator it;
	int count = 1;
	for(it = i_aimSettings->mapAwayMessages->begin(); 
			it != i_aimSettings->mapAwayMessages->end(); ++it){
		if(count == num){
			awayWidget->setText(it.data());
			awayMessage = it.data();
			if( awayRightNow == false){
				QString message = it.data();	
				if (message.length() > 1000){
				  message = message.mid(0, 950);
				  message = message + "... Im me for the rest of this away message.";
				}
				connection->setAwayMessage(message);
			}
			break;
		} else {
			count++;
		}
	}
  if ((num != 0) && (num == count)){
    awayWidget->show();
    if (awayRightNow == false){
      awayRightNow = true;
	emit(set_window(width(), height()-1) );
    }	
  }
}

/***************************************************************************
 * Set away = false.  Kinkatta autoresponds.
 ***************************************************************************/
void BuddyListWindow::away_BackForReal(){
  awayRightNow = false;
  awayWidget->hide();
  connection->setAwayMessage(QString(""));
  resizeEvent(0);
}


/**
 *
 *
 * The chatWindow Functions.
 *
 *
 */

/***************************************************************************
 * Send an IM message.  If the room is not equal to 0 then you want to send
 * to a chatroom.
 ***************************************************************************/
void BuddyListWindow::chat_sendImOut(QString message, QString target, int){
  TWINDOW_AIM_SANITY_CHECK;
  connection->sendIM(message, target, false, false);
}
/***************************************************************************
 * A message came in. Open window and send massage
 ***************************************************************************/
void BuddyListWindow::receivedIM(QString name, QString message, bool isAuto){
  Chat *targetWindow = messageWindow(name);
  targetWindow->show();
  // Slot autoin
  if (isAuto){
    message = QString("<B>Auto Response: </B>") + message;
  }
  targetWindow->updateBuddyName(name);
  targetWindow->messageIn(message);
  if (awayRightNow == true){
    if (targetWindow->sentAwayMessageToBuddy == false){
      
      QString outMessage = awayMessage;
      QString currentDate = QDate::currentDate().toString();
      QString currentTime = QTime::currentTime().toString();
      outMessage.replace( QRegExp("%d"), currentDate );
      outMessage.replace( QRegExp("%t"), currentTime );
      outMessage.replace( QRegExp("%n"), name );
		
      targetWindow->awayMessage(outMessage);
      connection->sendIM(awayMessage, name, true, true);
      targetWindow->sentAwayMessageToBuddy = true;
			
      awayMessageSent += new QString(name);
      QTimer::singleShot( POST_AWAY_MESSAGE_SENT_DELAY *1000, this, SLOT(postAwayMessageSent()));
    }
  }    
}

/***************************************************************************
 * Open a chat window with buddyName
 ***************************************************************************/
void BuddyListWindow::chatWindow_ChatWithBuddy(QString buddyName){
  Chat *targetWindow = messageWindow(buddyName);
  if (targetWindow){
    targetWindow->show();
  }
}

/***************************************************************************
 * Check what name is wanted to chat with
 ***************************************************************************/
void BuddyListWindow::chatWindow_ChatWithBuddy(){
  if (treeList->currentItem() == 0)
    treeList->setCurrentItem(treeList->firstChild());    
  QString name = "";
  QListViewItem  *cItem = treeList->currentItem();
  if ( cItem != NULL)
    name = treeList->currentName();
  if (treeList->checkIfFolder(cItem) == true)
    name = "";
 
  NameDialog named(this, "name dialog", true);
  QString text = named.go();
  if( !text.isEmpty() )
    chatWindow_ChatWithBuddy( text );
}


/***************************************************************************
 * Get a Chat Window
 ***************************************************************************/
Chat *BuddyListWindow::messageWindow(const QString &name){
  QString ChatWindowName = i_aimSettings->login_name + tocNormalize(name);
  Chat *targetWindow = messageWindows[ ChatWindowName ];
  if(targetWindow == NULL){
    TBuddyList list = connection->buddyList();
    if (list.getNum(name) == -1)
     	targetWindow = new Chat(i_aimSettings->login_name, name, false);
    else
      targetWindow = new Chat(i_aimSettings->login_name, name, true);
    messageWindows.insert(ChatWindowName, targetWindow);
    targetWindow->windows = windows;
    targetWindow->addWindowsMenu();
    fillUpWindows();
    connect(targetWindow, SIGNAL(messageOut(QString, QString,int)), this, SLOT(chat_sendImOut(QString, QString,int)) );
    messageWindows.setAutoDelete(false);
    targetWindow->browser = browser;
    targetWindow->updateChatSettings();
    targetWindow->updateChatSoundSettings();
    targetWindow->updateChatToolBarSettings();
    targetWindow->show();
    // Signals out from ChatWindow
    connect(targetWindow, SIGNAL(addBuddy(QString)), this, SLOT(slotAddBuddy(QString)));
    connect(targetWindow, SIGNAL(removeBuddy(QString)), this, SLOT(slotRemoveBuddy(QString)));
    connect(targetWindow, SIGNAL(blockBuddy(QString)), this, SLOT(chatWindow_BlockBuddy(QString)));
    connect(targetWindow, SIGNAL(warningOut(QString)), this, SLOT(chatWindow_WarnBuddy(QString)) );
    connect(targetWindow, SIGNAL(getInfo(QString)), this, SLOT(slotGetBuddyInfo(QString)) );
    connect(targetWindow, SIGNAL(saveToolbarSettings()), this, SIGNAL(saveSettings()));
    // Update Singles to Chat Window
    connect(this, SIGNAL(updateChatSettings()), targetWindow, SLOT(updateChatSettings()));
    connect(this, SIGNAL(updateChatSoundSettings()), targetWindow, SLOT(updateChatSoundSettings()));
    connect(this, SIGNAL(updateChatToolBarSettings()), targetWindow, SLOT(updateChatToolBarSettings()));
    connect(this, SIGNAL(updateIfIsBuddy(QString,bool)), targetWindow, SLOT(setIfIsBuddy(QString, bool)));

    // Quit signal from Chat Window
    connect(targetWindow, SIGNAL(quiting(QString, int)), this, SLOT(chatWindow_RemoveWindow(QString, int)));
  }

  return targetWindow;
}

/***************************************************************************
 * When a buddy window dies it is removed
 ***************************************************************************/
void BuddyListWindow::chatWindow_RemoveWindow(QString name, int){
  QString normedName = i_aimSettings->login_name + tocNormalize(name);
  Chat *targetWindow = NULL;
  targetWindow = messageWindows[ normedName ];
  if (targetWindow != NULL){
    messageWindows.remove(normedName);
    delete targetWindow;
  }
  fillUpWindows();
}

/******************************************************************************
 * block buddy "name"
 * @Peram name - Name of the buddy that is to be Blocked
 *****************************************************************************/
void BuddyListWindow::chatWindow_BlockBuddy(QString name){
  QString buddyName = "";
  int answer = QMessageBox::warning(0, APP_NAME " - Block Buddy",
    "Do you wish to block this buddy\nfrom your buddylist?",
    "Yes", "Cancel", 0,1);
  if (!answer){
    TBuddyList list = connection->denyList();
    TBuddy *edited = new TBuddy();
    edited->name = name;
    edited->group = 0;
    edited->status = TAIM_OFFLINE;
    edited->userClass = 0;
    edited->signonTime = 0;
    edited->evil = 0;
    list.add(edited);
    delete edited;
    connection->setDenyList(list);
    //connection->setServerBuddyList(list);
    //loadList();
    if(connection->permissions() != 4)
      connection->setPermissions(4);
  }	
}

/******************************************************************************
 * Prompt if you really want to warn "name"
 * Prompt if you want to do in anon or not
 * @Peram name - Name of the buddy that is to be Warned
 ******************************************************************************/
void BuddyListWindow::chatWindow_WarnBuddy(QString name){
  QString buddyName = "";
  int answer = QMessageBox::warning(0, APP_NAME " - Warn",
    QString("Do you wish to warn ") + name + "?",
    "Yes", "Cancel", 0,1);
  if (!answer){
    int anon = QMessageBox::warning(this, APP_NAME " - Message",
     "Do you wish to warn anonymously?",
     "Yes", "No", 0,1);
    TWINDOW_AIM_SANITY_CHECK;
    if (!anon)
      connection->sendWarning(name, true);
    else
      connection->sendWarning(name, false);
  }	
}







/**
 *
 *
 * The Import/Export Functions.
 *
 *
 */
/*
// Import buddy list from a buddy list.
void BuddyListWindow::importConfig(QString fileName){
  qApp->setOverrideCursor(Qt::waitCursor);
  // Got a file, open and parse.
  QFile file(fileName);
  QString config;
  if (file.open(IO_ReadWrite)){
    config += "CONFIG:";
    QTextStream stream(&file);
    while(!stream.eof()){
      config += stream.readLine();
      config += "\n";
    }
    file.close();
  }
  else
    QMessageBox::information(this, APP_NAME " - Error","Unable to open file.", QMessageBox::Ok);
  // Got config from file, parse out lists 
  TBuddyList buddyList = connection->buddyList();
  TBuddyList permitList = connection->permitList();
  TBuddyList denyList = connection->denyList();
  int permitStatus = connection->getStatus();
  tocParseConfig(config, &buddyList, &permitList, &denyList, &permitStatus);
 
	connection->setBuddyList(buddyList);
  connection->setPermitList(permitList);
  connection->setDenyList(denyList);
  emit ( saveSettings());
  qApp->restoreOverrideCursor();
}

// * Import buddy list from a buddy list.
void BuddyListWindow::importConfigAlias(QString fileName){
  qApp->setOverrideCursor(Qt::waitCursor);
	// Got a file, open and parse.
  QFile file(fileName);
  QString config;
  if (file.open(IO_ReadWrite)){
    config += "CONFIG:";
    QTextStream stream(&file);
    while(!stream.eof()){
      config += stream.readLine();
      config += "\n";
    }
    file.close();
  }
  else
    QMessageBox::information(this, APP_NAME " - Error","Unable to open file.", QMessageBox::Ok);
  // Got config from file, parse out lists 
  TBuddyList buddyList = connection->buddyList();
  TBuddyList permitList = connection->permitList();
  TBuddyList denyList = connection->denyList();
  int permitStatus = connection->getStatus();
  tocParseConfigAlias(config, &buddyList, &permitList, &denyList, &permitStatus);
  connection->setBuddyList(buddyList);
  connection->setPermitList(permitList);
  connection->setDenyList(denyList);
  emit ( saveSettings());
	qApp->restoreOverrideCursor();
}

// * Import buddy list from a gaim buddy list.
void BuddyListWindow::slotImportFromGaim(){
  QString buddyFile = getenv("HOME");
  buddyFile += "/.gaim";
  QString s( QFileDialog::getOpenFileName(buddyFile,QString("*.blist"),this, "Gaim Buddy List (*.blist)") );
  if ( s.isEmpty() == true)
  return;
  importConfigAlias(s);
}

// * Import buddy list from a kinkatta buddy list.
void BuddyListWindow::slotImportFromKinkatta(){
  QString buddyFile = getenv("HOME");
  buddyFile += "/." APP_LOWERCASE_NAME "/";
  QString s( QFileDialog::getOpenFileName(buddyFile,QString("*.xml *.buddyList"),this, APP_NAME " Buddy List (*.buddyList)") );
  if ( s.isEmpty() == true)
    return;
	  importConfigAlias(s);
}

// * Import buddy list from a tik buddy list.
void BuddyListWindow::slotImportFromTik(){
  QString buddyFile = getenv("HOME");
  buddyFile += "/.tic";
  QString s( QFileDialog::getOpenFileName(buddyFile,QString("*.config"),this, "Tik Buddy List (*.config)") );
  if ( s.isEmpty() == true)
    return;
  importConfig(s);
}

// * Export the buddylist to a file. 
void BuddyListWindow::slotExportBuddyList(){
  QString saveAsThisFile( QFileDialog::getSaveFileName( QString(""),QString("*.buddylist"),this, "buddyList (*.buddylist)") );
  
	if ( saveAsThisFile.isEmpty())
		return;
	QFileInfo fi(saveAsThisFile);
	if( fi.extension( FALSE ).isEmpty() )
		saveAsThisFile.append( ".buddylist" );

	qApp->setOverrideCursor(Qt::waitCursor);
  QFile file(saveAsThisFile);
  if (file.open(IO_ReadWrite))
  {
    QString config = setup::instance()->buddyListToString(i_aimSettings->login_name);
		QTextStream stream(&file);
		stream << config << "\n";		
    file.close();
  }
	else
		QMessageBox::critical(this, APP_NAME " - Message",("Could not export buddy list."));
	qApp->restoreOverrideCursor();  
}
*/

// BuddyListWindow



