/***************************************************************************
                              buddylistwindow.h
                             -------------------
    begin                : Sun Feb 13 2000
    copyright            : (C) 2000 by Benjamin Meyer
    email                : benjamin@csh.rit.edu
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/***************************************************************************
 * The main window that displays the buddies.  Edit/IM etc.
 ***************************************************************************/

#ifndef BUDDYLISTWINDOW_H
#define BUDDYLISTWINDOW_H

#include <qwidget.h>

#include "kinkattaviewlist.h"
#include <qdict.h>

#include "setup.h"
#include "awaymessage.h"
#include "tbuddylist.h"
#include "taim.h"
#include "chat.h"

class QMenuBar;
class QPopupMenu;
class QPushButton;

class Browser;

/**
  * @author Benjamin Meyer
  */

class BuddyListWindowItem; 

class BuddyListWindow : public QWidget  {
  Q_OBJECT

  signals:
    void logOff();
    void showOptions();
    void showAwayOptions();
    void set_window(int, int);
    void saveSettings();
    void quit();
	
    void updateIfIsBuddy(QString, bool);
    void updateChatSettings();

    void updateChatColorSettings();
    void updateChatSoundSettings();
    void updateChatToolBarSettings();

  public:
    BuddyListWindow(QWidget *parent=0, const char *name=0, TAim *i_connection=0 );
    void closeExistingChatwindows();
    void resizeEvent ( QResizeEvent *e );
    bool awayRightNow;
    QValueList<QString *> signedOff;
    QValueList<QString *> signedOn;
    QValueList<QString *> awayMessageSent;
    void clearPostConnections(void);
    Browser *browser;
    QPopupMenu *windows;
    void initMenuBar();
    void fillUpWindows();

  private:
    // internal vars
    settings_t *i_aimSettings;
    TAim *connection;
    bool viewAll;
    bool autoResizing;
    bool muteSound;  //whether or not to play sounds.  used at signon.
    bool loadingTheList;

    // Away status
    QString awayMessage;
		
    // internal functions
		void addTreeBuddy(TBuddy *buddy, QString folder, int folder_size);

    void initWindow();
    void initAwayList();
    void initViewAllSetting();
    void loadTreeBuddy(TBuddy *buddyInfo);
    void autoResize();
			 	
    QPopupMenu *_FileMenu;
    QPopupMenu *_ToolsMenu;
    QPopupMenu *_ViewMenu;
    QPopupMenu *_importMenu;
    QPopupMenu *_AwayMenu;
    QPopupMenu *_BuddyMenu;

    int id_buddyList;
    QPopupMenu *_FolderMenu;
    int id_folderEdit;
    int id_folderRemove;
    QPopupMenu *_HelpMenu;
    QMenuBar *_menuBar;

    QPopupMenu *_TreeMenu; 
    void loadSettings();
    
    // ChatWindow handling
    Chat *messageWindow(const QString &name);
    QDict<Chat> messageWindows;

    //void importConfig(QString fileName);
    //void importConfigAlias(QString fileName);

  protected:
    void mouseReleaseEvent(QMouseEvent* event);
    QPixmap *createBuddyPixmapWithIcon(QPixmap, int, QColor, QColor, QString);
		
    KinkattaListView *treeList;
    QPushButton *chatButton;
    QPushButton *infoButton;
    QPushButton *awayButton;
    
    AwayMessage *awayWidget;

  public slots:
    void rightClickInTree(QListViewItem *item, const QPoint &point, int);
    void buddyChanged(int number);
    void settingsChanged();
    void initOptions();
    void endMutePeriod();
    void loadList();
   void away_BackForReal();
   void receivedIM(QString name, QString message, bool isAuto);

  private slots:
    void postBuddySignoff();
    void postBuddySignon(); 
    void moveBuddy(QListViewItem* buddy, QListViewItem* folder);
    void aboutToShowFileMenu();
    void DoubleClicked();
    void TreeSelected(QListViewItem* selected);
    void folderExpanded(QListViewItem*item);
    void folderCollapsed(QListViewItem*item);

    void MenuSlotAbout();
    void slotAddBuddy();
    void slotAddBuddy(QString buddy);
    void slotRemoveBuddy();
    void slotAddFolder();
    void slotRemoveFolder();
    void slotRemoveFolder(QString group);
    void slotEditBuddy();
    void slotEditAlias();
    void changeBuddyName();
    void buddySignedOn(QString buddy); 
    void viewLastOn();
    void viewIdle();
		void ChangeViewAllSetting();
    void slotRemoveBuddy(QString name);

    void slotGetBuddyInfo();
    void slotGetBuddyInfo(QString name);
    void slotBuddyInfoRecieved(QString);

    void postAwayMessageSent();
    void away_ShowAwayList();
    void away_makeAwayNow(QString name);
    void away_Update(QString text);
    void away_GoAway(int);
		
    void chat_sendImOut(QString target, QString message, int room);
		
    void chatWindow_ChatWithBuddy(QString buddyName);
    void chatWindow_ChatWithBuddy();
    void chatWindow_RemoveWindow(QString name, int);
    void chatWindow_BlockBuddy(QString name);
    void chatWindow_WarnBuddy(QString name);

 /*
    void slotImportFromGaim();
    void slotImportFromKinkatta();
    void slotImportFromTik();
    void slotExportBuddyList();
 */
};

#endif

// buddylistwindow.h


