/******************************************************************************
					                        chat.cpp
					                   -------------------
		begin                : Mon May 8 2000
		copyright            : (C) 2000 by Benjamin Meyer
		email                : ben@meyerhome.net
 ******************************************************************************/

#include "chat.h"
#include "application.h"
#include "smiliespopup.h"

#include <qlayout.h>
#include <qfiledialog.h>
#include <qmsgbox.h>
#include <qdatetime.h>
#include <qclipboard.h>
#include <qpixmap.h>
#include <qaccel.h>
#include <qfont.h>
#include <qmenubar.h>
#include <qapplication.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qmultilineedit.h>
#include <qpopupmenu.h>
#include <qcolor.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qnamespace.h> 
#include <qpe/resource.h>
#include <qpe/config.h>
#include <qstylesheet.h>

/*****************************************************************/
// File-menu entries
#define ID_FILE_LOG                 10010
#define ID_FILE_SOUND               10020

/*****************************************************************/
// Edit-menu entries
#define ID_EDIT_UNDO                11010
#define ID_EDIT_REDO                11020

/*****************************************************************/
// View-menu entries
#define ID_VIEW_TIME                12010
#define ID_VIEW_CHATTOOLBAR         12020
#define ID_VIEW_TEXTTOOLBAR         12030

/*****************************************************************/
// Buudy-menu entries
#define ID_BUDDY_REMOVE             14040

/******************************************************************************
 * Constructor
 ******************************************************************************/
Chat::Chat(QString userName, QString buddysName,  bool buddy, QWidget *parent): QMainWindow(parent,buddysName){
  myName = userName;
	buddyName = buddysName;
	name = buddyName;
 
	chatSoundMuteButton = NULL;
	chatLogButton = NULL;
	chatTimeButton = NULL;
	
	sentAwayMessageToBuddy = false; 
	fontSize = 3;
	lastMessage = "";
	currentMessageOut = "";
	
	setCaption(buddyName + QString(" - " APP_NAME));

	chatSettings = setup::settings()->chatSettings;
	toolBarSettings = setup::settings()->chatToolBarSettings;
	soundSettings = setup::settings()->chatSoundSettings;
	
	//These are the only settings that can be changed from this chat window, so
	//we save them seperately.
	toolBarSettings_chat_log = false;
	soundSettings_chat_sound = soundSettings->chat_sound;
	soundSettings_chat_sound_beepReceive = soundSettings->chat_sound_beepReceive;
	soundSettings_chat_sound_beepSend = soundSettings->chat_sound_beepSend;
	chatSettings_chat_show_time = chatSettings->chat_show_time;

	fileMenu=new QPopupMenu(this, "FileMenu");
	editMenu=new QPopupMenu(this, "EditMenu");
	viewMenu=new QPopupMenu(this, "ViewMenu");
	buddyMenu = new QPopupMenu(this, "BuddyMenu");
	insertMenu = new QPopupMenu(this, "InsertMenu");
	insertFaceMenu = new QPopupMenu(this, "FaceMenu");
	connect(insertFaceMenu, SIGNAL(activated(int)), this, SLOT(slotInsertFace(int)));
					
	// MENUBAR CONFIGURATION
	// set menuBar() the current menuBar
	menuBar()->insertItem("&File", fileMenu);
	menuBar()->insertItem("&Edit", editMenu);
	menuBar()->insertItem("&View", viewMenu);
	menuBar()->insertItem("&Insert", insertMenu);
	menuBar()->insertItem("&Buddy", buddyMenu);
	initChatWindow();
	// Create Tool Bars
        fileToolbar = new QToolBar(this, "file operations");
	textToolbar = new QToolBar(this, "text operations");
	// Fill Tool bars
	setIcon(Icon("kinkatta_small"));
  isBuddy = buddy;
  initToolBar();
  initMenuBar();
  showMaximized();
}

void Chat::addWindowsMenu(){
  menuBar()->insertItem("|",1000);
  menuBar()->setItemEnabled(1000,false);
  menuBar()->insertItem("&Windows", windows);
}

/******************************************************************************
 * Deconstructor
 ******************************************************************************/
Chat::~Chat(){
  if (chatSettings_chat_log == true)
    logFile.close();
}

/******************************************************************************
 * Load the new settings
 ******************************************************************************/
void Chat::updateChatSettings(){
  chatSettings = setup::settings()->chatSettings;

  if (chatSettings_chat_show_time != chatSettings->chat_show_time)
    slotViewTimeStamp(); 
}

/******************************************************************************
 * Load the toolbar with the new settings
 ******************************************************************************/
void Chat::updateChatToolBarSettings(){
	//qDebug("Chat::updateChatToolBarSettings\n");
	toolBarSettings = setup::settings()->chatToolBarSettings;

	// View Text
	if ( toolBarSettings->view_text == true) {
		textToolbar->show();
		viewMenu->setItemChecked(ID_VIEW_TEXTTOOLBAR, true);
	}
	else{
		textToolbar->hide();
		viewMenu->setItemChecked(ID_VIEW_TEXTTOOLBAR, false);
	}
	// View Chat
	if ( toolBarSettings->view_chat == true) {
		fileToolbar->show();
		viewMenu->setItemChecked(ID_VIEW_CHATTOOLBAR, true);
	}
	else{
		fileToolbar->hide();
		viewMenu->setItemChecked(ID_VIEW_CHATTOOLBAR, false);
	}
	
	initToolBar();
	// scroll to the current text
	chatWindow->ensureVisible(0, chatWindow->contentsHeight());
}

/******************************************************************************
 * Load the new settings
 ******************************************************************************/
void Chat::updateChatSoundSettings(){
	soundSettings = setup::settings()->chatSoundSettings;

	//soundSettings_chat_sound = soundSettings->chat_sound;
	soundSettings_chat_sound_beepReceive = soundSettings->chat_sound_beepReceive;
	soundSettings_chat_sound_beepSend = soundSettings->chat_sound_beepSend;

	if (soundSettings_chat_sound != soundSettings->chat_sound)
		slotChatSound();
}

/******************************************************************************
 * MenuBar setup
 ******************************************************************************/
void Chat::initMenuBar(){
	// menuBar entry fileMenu
	fileMenu->clear();
	fileMenu->insertItem(Icon("log"), "&Log", this, SLOT(slotChatLog()), CTRL+Key_L, ID_FILE_LOG);
	fileMenu->insertItem(Resource::loadPixmap("bell"), "&Sound", this, SLOT(slotChatSound()), 0, ID_FILE_SOUND);
	fileMenu->insertItem(Resource::loadPixmap("txt"), "&Clear", this, SLOT(slotChatClear()), CTRL+Key_D, -1);
	fileMenu->insertSeparator();
	fileMenu->insertItem(Resource::loadPixmap("close"), "Close", this, SLOT(slotChatClose()), CTRL+Key_Q, -1);

	if (chatSettings_chat_log == true)
		fileMenu->setItemChecked(ID_FILE_LOG, true);
	else
		fileMenu->setItemChecked(ID_FILE_LOG, false);

	if (soundSettings_chat_sound == true)
		fileMenu->setItemChecked(ID_FILE_SOUND, true);
	else
		fileMenu->setItemChecked(ID_FILE_SOUND, false);

	// menuBar entry editMenu
	editMenu->clear();
	editMenu->insertItem(Resource::loadPixmap("back"), "U&ndo", inputWindow, SLOT(undo()), CTRL+Key_N, ID_EDIT_UNDO);
	editMenu->insertItem(Resource::loadPixmap("forward"), "R&edo", inputWindow, SLOT(redo()), CTRL+Key_E, ID_EDIT_REDO);
	editMenu->insertSeparator();
	editMenu->insertItem(Resource::loadPixmap("cut"), "Cu&t", inputWindow, SLOT(cut()), CTRL+Key_X, 0);
	editMenu->insertItem(Resource::loadPixmap("copy"), "&Copy", this, SLOT(slotEditCopy()), CTRL+Key_C, 0);
	editMenu->insertItem(Resource::loadPixmap("paste"), "&Paste", inputWindow, SLOT(paste()), CTRL+Key_V, 0);
	editMenu->insertSeparator();
	editMenu->insertItem("&Select All", this, SLOT(slotEditSelectAll()), CTRL+Key_A, 0);

	editMenu->setItemEnabled ( ID_EDIT_UNDO, false );
	editMenu->setItemEnabled ( ID_EDIT_REDO, false );

	// menuBar entry viewMenu
	viewMenu->clear();
	viewMenu->setCheckable(true);
	viewMenu->insertItem(Icon("chat_time"), "&Time Stamp", this, SLOT(slotViewTimeStamp()), Key_F2, ID_VIEW_TIME);
	viewMenu->insertSeparator();
	viewMenu->insertItem("&Chat Toolbar", this, SLOT(slotViewChatToolBar()), 0, ID_VIEW_CHATTOOLBAR);
	viewMenu->insertItem("Text Tool&bar", this, SLOT(slotViewTextToolBar()), 0, ID_VIEW_TEXTTOOLBAR);
	viewMenu->setItemChecked(ID_VIEW_CHATTOOLBAR, true);
	viewMenu->setItemChecked(ID_VIEW_TEXTTOOLBAR, true);

	viewMenu->setItemChecked(ID_VIEW_TIME, true);

	insertMenu->clear();
	insertMenu->insertItem(Icon("face_bsmile"), "&Smilies",  insertFaceMenu);
	insertMenu->insertSeparator();
	insertFaceMenu->clear();
	insertFaceMenu->insertItem(Icon("face_smile"), " " /* ":-)" */,  FACE_SMILE_ID);
	insertFaceMenu->setAccel(CTRL + Key_1, FACE_SMILE_ID);
	insertFaceMenu->insertItem(Icon("face_sad")," " /* ":-(" */, FACE_SAD_ID);
	insertFaceMenu->setAccel(CTRL + Key_2, FACE_SAD_ID);
	insertFaceMenu->insertItem(Icon("face_wink")," " /* ";-)" */, FACE_WINK_ID);
	insertFaceMenu->setAccel(CTRL + Key_3, FACE_WINK_ID);
	insertFaceMenu->insertItem(Icon("face_tongue")," " /* ":-P" */, FACE_TONGUE_ID);
	insertFaceMenu->setAccel(CTRL + Key_4, FACE_TONGUE_ID);

	insertFaceMenu->insertItem(Icon("face_scream")," " /* "=-O" */, FACE_SCREAM_ID);
	insertFaceMenu->setAccel(CTRL + Key_5, FACE_SCREAM_ID);
	insertFaceMenu->insertItem(Icon("face_kiss")," " /* ":-*" */,  FACE_KISS_ID);
	insertFaceMenu->setAccel(CTRL + Key_6, FACE_KISS_ID);
	insertFaceMenu->insertItem(Icon("face_yell")," " /* ">:o" */, FACE_YELL_ID);
	insertFaceMenu->setAccel(CTRL + Key_7, FACE_YELL_ID);
	insertFaceMenu->insertItem(Icon("face_glasses")," " /* "8-)" */, FACE_GLASSES_ID);
	insertFaceMenu->setAccel(CTRL + Key_8, FACE_GLASSES_ID);

	insertFaceMenu->insertItem(Icon("face_oneeye")," " /* "O-)" */, FACE_ONEEYE_ID);
	insertFaceMenu->setAccel(CTRL + Key_9, FACE_ONEEYE_ID);

	insertFaceMenu->insertItem(Icon("face_moneymouth")," " /* ":-$" */, FACE_MONEYMOUTH_ID);
	insertFaceMenu->setAccel(ALT + Key_1, FACE_MONEYMOUTH_ID);
	insertFaceMenu->insertItem(Icon("face_burp")," " /* ":-!" */,  FACE_BURP_ID);
	insertFaceMenu->setAccel(ALT + Key_2, FACE_BURP_ID);
	insertFaceMenu->insertItem(Icon("face_embarrassed")," " /* ":-[" */, FACE_EMBARRASSED_ID);
	insertFaceMenu->setAccel(ALT + Key_3, FACE_EMBARRASSED_ID);
	insertFaceMenu->insertItem(Icon("face_angel")," " /* "O:-)" */, FACE_ANGEL_ID);
	insertFaceMenu->setAccel(ALT + Key_4, FACE_ANGEL_ID);

	insertFaceMenu->insertItem(Icon("face_think")," " /* ":-\\" */, FACE_THINK_ID);
	insertFaceMenu->setAccel(ALT + Key_5, FACE_THINK_ID);
	insertFaceMenu->insertItem(Icon("face_cry")," " /* ":'(" */,  FACE_CRY_ID);
	insertFaceMenu->setAccel(ALT + Key_6, FACE_CRY_ID);
	insertFaceMenu->insertItem(Icon("face_crossedlips")," " /* ":-X" */, FACE_CROSSEDLIPS_ID);
	insertFaceMenu->setAccel(ALT + Key_7, FACE_CROSSEDLIPS_ID);
	insertFaceMenu->insertItem(Icon("face_bigsmile")," " /* ":-D" */, FACE_BIGSMILE_ID);
	insertFaceMenu->setAccel(ALT + Key_8, FACE_BIGSMILE_ID);

	insertFaceMenu->insertItem(Icon("face_luke")," " /* "C:-)" */, FACE_LUKE_ID);
	insertFaceMenu->setAccel(ALT + Key_9, FACE_LUKE_ID);

	insertMenu->insertItem(Resource::loadPixmap("bold"), "&Bold", this, SLOT(slotInsertBold()), CTRL+Key_B, 0);
	insertMenu->insertItem(Resource::loadPixmap("italic"), "&Italic", this, SLOT(slotInsertItalic()), CTRL+Key_I, 0);
	insertMenu->insertItem(Resource::loadPixmap("underline"), "&Underline", this, SLOT(slotInsertUnderline()), CTRL+Key_U, 0);
	insertMenu->insertSeparator();
	insertMenu->insertItem(Icon("small"), "S&hrink Font", this, SLOT(slotInsertShrinkFont()), CTRL+Key_Minus, 0);
	insertMenu->insertItem(Icon("normal"), "&Normal Font", this, SLOT(slotInsertNormalFont()), 0, 0);
	insertMenu->insertItem(Icon("big"), "In&crease Font", this, SLOT(slotInsertIncreaseFont()), CTRL+Key_Plus, 0);
	insertMenu->insertSeparator();
	insertMenu->insertItem("L&ast Message", this, SLOT(slotInsertLastMessage()),Key_Insert, 0);

	buddyMenu->clear();
	buddyMenu->insertItem(Icon("send"), "&Send Message", this, SLOT(slotBuddySendMessage()), CTRL+Key_S, 0);
  buddyMenu->insertItem(Icon("info"), "&Buddy Info", this, SLOT(slotBuddyInfo()), 0, 0);
  
  buddyMenu->insertItem(Icon("warn"), "&Warn Buddy...", this, SLOT(slotBuddyWarn()), 0, 0);
  if (isBuddy == true)
    buddyMenu->insertItem(Icon("remove"), "&Remove Buddy...", this, SLOT(slotBuddyAddRemove()), 0, ID_BUDDY_REMOVE);
  else
    buddyMenu->insertItem(Icon("add"), "&Add Buddy", this, SLOT(slotBuddyAddRemove()), 0, ID_BUDDY_REMOVE);
  
  buddyMenu->insertItem(Icon("block"), "B&lock Buddy...", this, SLOT(slotBuddyBlock()), 0, 0);
		
}

/******************************************************************************
 * ToolBar setup  Clears the toolbars and fills them with the widgets
 * specified in toolBarSettings then oriants the toolbars as chosen.
 ******************************************************************************/
void Chat::initToolBar(){
  //Config prefs(APP_NAME);
  //prefs.setGroup("sound");

  // File Tool Bar
  //fileToolbar->setVerticalStretchable(true);
  //fileToolbar->setOrientation (Vertical);
  fileToolbar->hide();
  fileToolbar->clear();
  chatClearButton = new QToolButton(Resource::loadPixmap("txt"), "Clear Chat", 0, this,
                                          SLOT(slotChatClear()), fileToolbar);
  fileToolbar->addSeparator();

  chatTimeButton = new QToolButton(Icon("chat_time"), "Show Time", 0, this,
                                          SLOT(slotViewTimeStamp()), fileToolbar);
  
  if (chatSettings_chat_show_time == true)
    if(chatTimeButton) chatTimeButton->setDown(true);
  else
    if(chatTimeButton) chatTimeButton->setDown(false);

  chatLogButton = new QToolButton(Icon("log"), "Log", 0, this, SLOT(slotChatLog()), fileToolbar);
  
  if (chatSettings_chat_log == true )
    if(chatLogButton) chatLogButton->setDown( true );
  else
    if(chatLogButton) chatLogButton->setDown( false );

    chatSoundMuteButton = new QToolButton(Resource::loadPixmap("bell"), "Sound", 0, this, SLOT(slotChatSound()), fileToolbar);

  
  if (soundSettings_chat_sound == true ){
      chatSoundMuteButton->setDown ( true );
      chatSoundMuteButton->setTextLabel ("Sound Enabled", true);
  }
  else{
      chatSoundMuteButton->setDown ( false );
      chatSoundMuteButton->setTextLabel ("Sound Muted", true);
  }
  fileToolbar->addSeparator();
    userInfoButton = new QToolButton(Icon("info"), "Buddy Info", 0, this,
                                          SLOT(slotBuddyInfo()), fileToolbar);
  
    if(isBuddy == true)
      userRemoveButton = new QToolButton(Icon("remove"), "Remove Buddy", 0, this, SLOT(slotBuddyAddRemove()), fileToolbar);
    else
      userRemoveButton = new QToolButton(Icon("add"), "Add Buddy", 0, this, SLOT(slotBuddyAddRemove()), fileToolbar);
    userBlockButton = new QToolButton(Icon("block"), "Block Buddy", 0, this,
                                          SLOT(slotBuddyBlock()), fileToolbar);

  sendMessageButton = new QToolButton(Icon("send"), "Send Message", 0, this,
                                          SLOT(slotBuddySendMessage()), fileToolbar);

  if (toolBarSettings->view_chat == true){
    if (toolBarSettings->loc_chat == 0){
      QMainWindow::moveToolBar(fileToolbar,QMainWindow::Top);
      fileToolbar->show();
    }
    if (toolBarSettings->loc_chat == 1){
      QMainWindow::moveToolBar(fileToolbar,QMainWindow::Right);
      fileToolbar->show();
    }
    if (toolBarSettings->loc_chat == 2){
      QMainWindow::moveToolBar(fileToolbar,QMainWindow::Bottom);
      fileToolbar->show();
    }
    if (toolBarSettings->loc_chat == 3){
      QMainWindow::moveToolBar(fileToolbar,QMainWindow::Left);
      fileToolbar->show();
    }
  }

  textToolbar->hide();
  // TextToolBar
  textToolbar->clear();
  textBoldButton = new QToolButton(Resource::loadPixmap("bold"), "Bold", 0, this, SLOT(slotInsertBold()), textToolbar);
    textItalicButton = new QToolButton(Resource::loadPixmap("italic"), "Italic", 0, this, SLOT(slotInsertItalic()), textToolbar);
    textUnderLineButton = new QToolButton(Resource::loadPixmap("underline"), "Underline", 0, this, SLOT(slotInsertUnderline()), textToolbar);
  
    textToolbar->addSeparator();
    textUpFontButton = new QToolButton(Icon("small"), "Decrease Font", 0, this,
                                          SLOT(slotInsertShrinkFont()), textToolbar);
    textNormFontButton = new QToolButton(Icon("normal"), "Normal Font", 0, this,
                                          SLOT(slotInsertNormalFont()), textToolbar);
    textDownFontButton = new QToolButton(Icon("big"), "Increase Font", 0, this,
                                          SLOT(slotInsertIncreaseFont()), textToolbar);
    textToolbar->addSeparator();

    smileyButton = new QToolButton(textToolbar, "smileyButton");
    smileyButton->setPixmap(Icon("face_bsmile"));
    smileyButton->setTextLabel(QString(":-D"));

    QPopupMenu *smileyPopupMenu = new QPopupMenu(this, "smileyPopupMenu");
    smileyButton->setPopup(smileyPopupMenu);
    smileyButton->setPopupDelay(0);

    Smilies *smileyPopup = new Smilies(smileyPopupMenu, "smileyPopup");
    smileyPopupMenu->insertItem(smileyPopup, 1, 1);
    connect(smileyPopup, SIGNAL(smileyButtonClicked(int)), this, SLOT(slotInsertFace(int)));

  if (toolBarSettings->view_text == true){
    if (toolBarSettings->loc_text == 0){
      QMainWindow::moveToolBar(textToolbar,QMainWindow::Top);
      textToolbar->show();
    }
    if (toolBarSettings->loc_text == 1){
      QMainWindow::moveToolBar(textToolbar,QMainWindow::Right);
      textToolbar->show();
    }
    if (toolBarSettings->loc_text == 2){
      QMainWindow::moveToolBar(textToolbar,QMainWindow::Bottom);
      textToolbar->show();
    }
    if (toolBarSettings->loc_text == 3){
      QMainWindow::moveToolBar(textToolbar,QMainWindow::Left);
      textToolbar->show();
    }
  }
}

/******************************************************************************
 * Sets up the input window and the chat window
 ******************************************************************************/
void Chat::initChatWindow(){
  bigOne = new QWidget(this, "MainWidget");
  setCentralWidget( bigOne );

  chatWindow = new KaimTextBrowser(bigOne, "ChatWindow");
  chatWindow->setHScrollBarMode(QScrollView::AlwaysOff);
  connect(chatWindow, SIGNAL(gotoURL(QString)), this, SLOT(urlClicked(QString)));

  inputWindow = new QMultiLineEdit( bigOne , "InputWindow");
  inputWindow->setWordWrap(QMultiLineEdit::WidgetWidth);
  connect(inputWindow, SIGNAL(undoAvailable(bool)), this, SLOT(slotEditTurnOnOffUndo(bool)));
  connect(inputWindow, SIGNAL(redoAvailable(bool)), this, SLOT(slotEditTurnOnOffRedo(bool)));
  connect(inputWindow, SIGNAL(textChanged()), this, SLOT(inputTextChanged()));
  connect(inputWindow, SIGNAL(copyAvailable(bool)), this, SLOT( copyChanged(bool)));
  postNumber = 0;
	
  connect(inputWindow, SIGNAL(returnPressed()), this, SLOT(enterKeyPressed()));

  Form1Layout = new QVBoxLayout( bigOne ); 
  Form1Layout->setSpacing( 3 );
  Form1Layout->setMargin( 0 );
 
  Form1Layout->addWidget( chatWindow );

  inputWindow->setMaximumSize( QSize( 32767, 40 ) );
  Form1Layout->addWidget( inputWindow );

  inputWindow->setFocus();
}

/******************************************************************************
* apply settings to inputWindow
******************************************************************************/
void Chat::applytoInputWindow(){
	QFont tempFont = qApp->font();
	QStyleSheet* tempStyle;

	// set to adjust font pointSize by equiv html size from up/down arrows in chat
	if (fontSize != 3) {
		if (chatWindow) {
			tempStyle = chatWindow->styleSheet();
			tempStyle->scaleFont(tempFont,fontSize);
		}
	}
	inputWindow->setFont(tempFont);
}

/******************************************************************************
 * The buddy signed off and so somthing should be written.
 ******************************************************************************/
void Chat::buddySignedOff(){
	QString CurrentTime = QTime::currentTime().toString().data();
	QString message = QString("<B>") + buddyName + " Signed off at: " + CurrentTime + "</B>\n";
	if (postNumber == 0)
		chatWindow->setText(message);
	else
		chatWindow->append(message);
	postNumber++;

	conversationBuffer += message;
	conversationTimeBuffer += message;

	chatWindow->ensureVisible(0, chatWindow->contentsHeight());
	// This should not be a hard coded value like this!!!
	chatWindow->repaintContents(0,0, 10000, 20000, true);

	// Add to the logs.
	if (chatSettings_chat_log == true)
	  logStream << buddyName + " Signed off at: " + CurrentTime + "\n";
}

/******************************************************************************
 * The buddy signed on and so somthing should be written.
 ******************************************************************************/
void Chat::buddySignedOn(){
	QString CurrentTime = QTime::currentTime().toString().data();
	QString message = QString("<B>") + buddyName + " Signed on at: " + CurrentTime + "</B>\n";
	if (postNumber == 0)
		chatWindow->setText(message);
	else
		chatWindow->append(message);
	postNumber++;

	conversationBuffer += message;
	conversationTimeBuffer += message;

	chatWindow->ensureVisible(0, chatWindow->contentsHeight());
	// This should not be a hard coded value like this!!!
	chatWindow->repaintContents(0,0, 10000, 20000, true);
	// Add to the logs.
	if (chatSettings_chat_log == true)
	  logStream << buddyName + " Signed off at: " + CurrentTime + "\n";
}

/******************************************************************************
 * When you send an Away message to someone put it on the chat window also.
 ******************************************************************************/
void Chat::awayMessage(QString message){
	message = QString("<B>") + "Auto Reply:</B> " + message +"\n";
	if (postNumber == 0)
		chatWindow->setText(message);
	else
		chatWindow->append(message);
	postNumber++;

	conversationBuffer += message;
	conversationTimeBuffer += message;

	chatWindow->ensureVisible(0, chatWindow->contentsHeight());
	// This should not be a hard coded value like this!!!
	chatWindow->repaintContents(0,0, 10000, 20000, true);
	// Add to the logs.
	if (chatSettings_chat_log == true)
	  logStream << buddyName + "Auto Reply: " + message + "\n";
}

/******************************************************************************
 * Sets the log variable
 ******************************************************************************/
void Chat::slotChatLog(){
	if (chatSettings_chat_log == false){
		chatSettings_chat_log = true;
		fileMenu->setItemChecked(ID_FILE_LOG, true);
		if (toolBarSettings_chat_log == true && chatLogButton != NULL){
			chatLogButton->setDown ( true );
		}
		openLogFile();
	}
	else{
		chatSettings_chat_log = false;
		fileMenu->setItemChecked(ID_FILE_LOG, false);
		if (toolBarSettings_chat_log == true && chatLogButton != NULL){
			chatLogButton->setDown ( false );
		}
		// Delete log file
		//qDebug("logFile.close() %s\n", logFile.name().latin1());
		logFile.close();
		if (logFile.remove() == false)
			QMessageBox::critical(0, APP_NAME, QString("Can not delete logfile "), "OK");
	}
}

/******************************************************************************
 * Clear the chat window
 ******************************************************************************/
void Chat::slotChatClear(){
  chatWindow->setText( "", chatWindow->context());
  postNumber = 0;
  // Clear conversation
  conversationBuffer = "Conversations with " + buddyName + "\n";
  conversationTimeBuffer = "Conversations with " + buddyName + "\n";
}

/******************************************************************************
 * Turn Sound on and off
 ******************************************************************************/
void Chat::slotChatSound(){
	if (soundSettings_chat_sound == false){
		soundSettings_chat_sound = true;
		fileMenu->setItemChecked(ID_FILE_SOUND, true);
		if (chatSoundMuteButton != NULL){
			chatSoundMuteButton->setDown ( true );
			chatSoundMuteButton->setTextLabel ("Sound Enabled", true);
		}
		if (soundSettings_chat_sound_beepReceive == false &&
				soundSettings_chat_sound_beepSend == false){
			soundSettings_chat_sound_beepReceive = true;
			soundSettings_chat_sound_beepSend = true;
		}
	}
	else{
		soundSettings_chat_sound = false;
		fileMenu->setItemChecked(ID_FILE_SOUND, false);
		if (chatSoundMuteButton != NULL){
			chatSoundMuteButton->setDown ( false );
			chatSoundMuteButton->setTextLabel ("Sound Muted", true);
		}
	}
}

/******************************************************************************
 * Close out of the Chat
 ******************************************************************************/
void Chat::closeEvent(QCloseEvent *e) {
  // Close window
  emit quiting(buddyName, -1);
  QMainWindow::closeEvent(e);
}

/******************************************************************************
 * Close out of the Chat
 ******************************************************************************/
void Chat::slotChatClose(){
  this->close();
}

/******************************************************************************
 * Copy highlighted to clipboard
 ******************************************************************************/
void Chat::slotEditCopy(){
  if (inputWindow->hasFocus())
    inputWindow->copy();
  else
   chatWindow->copy();
}

/******************************************************************************
 * Select All
 ******************************************************************************/
void Chat::slotEditSelectAll(){
  if (inputWindow->hasFocus())
    inputWindow->selectAll();
  else
    chatWindow->selectAll();
}

/******************************************************************************
 *  Enable/disable undo
 ******************************************************************************/
void Chat::slotEditTurnOnOffUndo(bool change){
	if (change == false)
		editMenu->setItemEnabled ( ID_EDIT_UNDO, false );
	else
		editMenu->setItemEnabled ( ID_EDIT_UNDO, true );
}

/******************************************************************************
 * Enable/disable redo
 ******************************************************************************/
void Chat::slotEditTurnOnOffRedo(bool change){
	if (change == false)
		editMenu->setItemEnabled ( ID_EDIT_REDO, false );
	else
		editMenu->setItemEnabled ( ID_EDIT_REDO, true );
}

/******************************************************************************
 * Toggle the Chat Toolbar
 ******************************************************************************/
void Chat::slotViewChatToolBar(){
	// turn Toolbar on or off
	if (fileToolbar->isVisible())
	{
		toolBarSettings->view_chat = false;
		fileToolbar->hide();
		viewMenu->setItemChecked(ID_VIEW_CHATTOOLBAR, false);
	}
	else
	{
		toolBarSettings->view_chat = true;
		fileToolbar->show();
		viewMenu->setItemChecked(ID_VIEW_CHATTOOLBAR, true);
	};
  emit saveToolbarSettings();
}

/******************************************************************************
 * Toggle the Text Toolbar
 ******************************************************************************/
void Chat::slotViewTextToolBar(){
	// turn Toolbar on or off
	if (textToolbar->isVisible())
	{
		textToolbar->hide();
		toolBarSettings->view_text = false;
		viewMenu->setItemChecked(ID_VIEW_TEXTTOOLBAR, false);
	}
	else
	{
		toolBarSettings->view_text = true;
		textToolbar->show();
		viewMenu->setItemChecked(ID_VIEW_TEXTTOOLBAR, true);
	};
  emit saveToolbarSettings();
}

/******************************************************************************
 * Toggle the TimeStamp Variable
 ******************************************************************************/
void Chat::slotViewTimeStamp(){
	if (chatSettings_chat_show_time == false){
		chatSettings_chat_show_time = true;
		viewMenu->setItemChecked(ID_VIEW_TIME, true);
		chatTimeButton->setDown ( true );
		chatWindow->setText(conversationTimeBuffer);
		if (chatSettings_chat_log == true){
			// Turn off
			slotChatLog();
			// And back on, but with time (in case)
			slotChatLog();
		}
	}
	else{
		chatSettings_chat_show_time = false;
		viewMenu->setItemChecked(ID_VIEW_TIME, false);
		chatTimeButton->setDown ( false );
		chatWindow->setText(conversationBuffer);
	}
	// scroll to the new text
	chatWindow->ensureVisible(0, chatWindow->contentsHeight());
}

/******************************************************************************
 * Insert one of those "emotions" into the window.  Boy do I hate these.
 * When done update the currentMessageOut sense no signal is emmited
 ******************************************************************************/
void Chat::slotInsertFace(int face){
#define doface(x, y) \
	if (face == x){ \
		inputWindow->insert(y); \
		currentMessageOut = inputWindow->text(); \
		return; \
	}

	doface(FACE_SMILE_ID, FACE_SMILE_TEXT);
	doface(FACE_SAD_ID, FACE_SAD_TEXT);
	doface(FACE_WINK_ID, FACE_WINK_TEXT);
	doface(FACE_TONGUE_ID, FACE_TONGUE_TEXT);
	doface(FACE_SCREAM_ID, FACE_SCREAM_TEXT);
	doface(FACE_KISS_ID, FACE_KISS_TEXT);
	doface(FACE_YELL_ID, FACE_YELL_TEXT);
	doface(FACE_GLASSES_ID, FACE_GLASSES_TEXT);
	doface(FACE_ONEEYE_ID, FACE_ONEEYE_TEXT);
	doface(FACE_MONEYMOUTH_ID, FACE_MONEYMOUTH_TEXT);
	doface(FACE_BURP_ID, FACE_BURP_TEXT);
	doface(FACE_EMBARRASSED_ID, FACE_EMBARRASSED_TEXT);
	doface(FACE_ANGEL_ID, FACE_ANGEL_TEXT);
	doface(FACE_THINK_ID, FACE_THINK_TEXT);
	doface(FACE_CRY_ID, FACE_CRY_TEXT);
	doface(FACE_CROSSEDLIPS_ID, FACE_CROSSEDLIPS_TEXT);
	doface(FACE_BIGSMILE_ID, FACE_BIGSMILE_TEXT);
	doface(FACE_LUKE_ID, FACE_LUKE_TEXT);

#undef doface
}

/******************************************************************************
 * Inserts a set of tags into the input window.
 ******************************************************************************/
void Chat::slotInsertTags(QString open, QString close){
	QString oldtext = "";
	if (canCopy){  
		// Cut any selected text to clipboard
		inputWindow->cut();
		// Copy text from the clipboard (paste)
		oldtext = QApplication::clipboard()->text();
	}
	// Add tags
	oldtext = open  + oldtext + close;
	// paste new text into input window
	inputWindow->insert(oldtext);
	int a,b;
	inputWindow->getCursorPosition(&a,&b);
	inputWindow->setCursorPosition(a,b-close.length(),false);
	currentMessageOut = inputWindow->text();
}

/******************************************************************************
 * Insert bold tags in the input window
 ******************************************************************************/
void Chat::slotInsertBold(){
	slotInsertTags( "<b>" , "</b>" );
}

/******************************************************************************
 * Insert italic tags in the input window
 ******************************************************************************/
void Chat::slotInsertItalic(){
	slotInsertTags( "<i>" , "</i>" );
}

/******************************************************************************
 * Insert underline tags in the input window
 ******************************************************************************/
void Chat::slotInsertUnderline(){
	slotInsertTags( "<u>" , "</u>" );
}

/******************************************************************************
 * Insert small tags in the input window
 ******************************************************************************/
void Chat::slotInsertShrinkFont(){
	if (fontSize > 0)
		fontSize--;
        applytoInputWindow();
}

/******************************************************************************
 * Set the Font back to normal
 ******************************************************************************/
void Chat::slotInsertNormalFont(){
	fontSize = normalFontSize;
        applytoInputWindow();
}

/******************************************************************************
 * insert font tags
 ******************************************************************************/
void Chat::slotInsertIncreaseFont(){
	if (fontSize < 7)
		fontSize++;
        applytoInputWindow();
}

/***************************************************************************
 * Puts the last message sent into the input box.
 ***************************************************************************/
void Chat::slotInsertLastMessage(){
	if (inputWindow->text() == QString(""))
		inputWindow->setText(lastMessage);
	else
		inputWindow->append(lastMessage);
}

/******************************************************************************
 * Send the message in the input window
 ******************************************************************************/
void Chat::slotBuddySendMessage(){
	//currentMessageOut = "";
	QString fontsizeTag = "";
	QString fontfaceTag = "";
	QString fontcolorTag = "";

	// Example Output HTML
	//<HTML><BODY BGCOLOR="#FFFF00"><FONT COLOR="#FF00FF">Room for one more?</FONT></BODY></HTML>
	//inputWindow->setCursorPosition(0, 0, true);
	bool send = true;
	// A blank QMultilineedit has 2 chars in it at start \n\n??
	if (inputWindow->length() == 0){
	  send = false;
	  QString message = inputWindow->text().data();
	  if(message == "\n")
	    inputWindow->setText( "" );
	}
	if (send == true){
		// Send the input window data
		QString message = inputWindow->text().data();
		inputWindow->setText( "" );
		// Save this message in case people want to use it.
		QString lastLetter = message.mid(message.length()-1,1);
		if (lastLetter == "\n"){
			message = message.mid(0, message.length()-1);
			lastMessage = message;
		}
		lastLetter = message.mid(message.length()-1,1);
		if (lastLetter == "\n"){
			message = message.mid(0, message.length()-1);
			lastMessage = message;
		}

		QRegExp url;
		QString link;
		int len = 0, start = 0, end = 0, endanchor = 0;
		int starthttp = 0, startwww = 0, startanchor = 0, skipto = 0;
		int lenhttp = 0, lenwww = 0;
		bool http;
		url.setCaseSensitive(FALSE);

		//qDebug("message %s\n", message.latin1());
		while(start != -1){
			//first we search for any already-inserted anchors, that would be here
			//from "Insert->Link" so as not to process them and leave them alone.
			url.setPattern("<a href");
			startanchor = url.match(message, end);
			if(startanchor != -1){
				url.setPattern("</a>");
				endanchor = url.match(message, startanchor, &len);
				skipto = endanchor + len;
				//qDebug("skipto %d end %d\n", skipto, end);
			}

			url.setPattern( "http://[a-zA-Z0-9][^ ^\n^\r^\t]*" );
			starthttp = url.match(message, end, &lenhttp);
			url.setPattern( "www\\.[a-zA-Z0-9][^ ^\n^\r^\t]*" );
			startwww = url.match(message, end, &lenwww);

			//qDebug("startanchor %d startwww %d starthttp %d\n", startanchor, startwww, starthttp);
			//qDebug("lenwww %d lenhttp %d\n", lenwww, lenhttp);

			if( (startanchor != -1)
				&& (startanchor < (starthttp > -1 ? starthttp : startanchor+1) )
				&& (startanchor < (startwww > -1 ? startwww : startanchor+1) ) ){
				end = skipto;
				//qDebug("SKIP skipto: %d\n", skipto);
				goto SKIP;
			}

			if( (starthttp == -1) && (startwww == -1) ) //neither found
				break;
			else if(starthttp == -1){ //http not found
				start = startwww;
				end = startwww + lenwww;
				http = FALSE;
			}else if(startwww == -1){ //www not found
				start = starthttp;
				end = starthttp + lenhttp;
				http = TRUE;
			} else { //both found. use the first one!
				start = (starthttp < startwww) ? starthttp : startwww;
				end = (starthttp < lenwww) ? starthttp + lenhttp : startwww + lenwww;
				http = (starthttp < lenwww) ? TRUE : FALSE;
			}

			//url.setPattern( QString("[ \n\r\t]") );  //e.g, any whitespace
			//end = url.match(message, start+len, 0, TRUE);

			//this happens when the damn end of the url is at the end of message
			if(end == -1)
				end = message.length();

			//insert this first, makes it easier.
			message.insert(end, "</a>");

			if(http)
				link = "<a href=\"" + message.mid(start, end-start) + "\">";
			else //need the leading "http://"
				link = "<a href=\"http://" + message.mid(start, end-start) + "\">";

			message.insert(start, link );
			end += link.length() + 4; // </a> == 4

			SKIP:
			 ; //pacifying g++

		}

		// render message to be sent with HTML in the chatwindow (TextBrowser widget)
		updateHTML(message, myName, "#003c80");

		// get the font tag...
		QString fontTag;
		fontTag = fontTagHTML(fontSize);

		// wrap it around the message...
		if(!fontTag.isEmpty()){
			fontTag = "<FONT" + fontTag + ">";
		 	message = fontTag + message + "</FONT>";
		}

		// fix for smily rendering in winAim
		message = fontFixSmilies(message, fontTag);

		message = QString("<HTML>") + message + "</HTML>";
		emit( messageOut( message, buddyName, 0 ) );
		// emit beep in wanted.
		if ( ( soundSettings_chat_sound == true ) &&
		   ( soundSettings_chat_sound_beepSend == true)){
			beep();
		}
	}
}

/***************************************************************************
 * Occurs when the message comes in
 ***************************************************************************/
void Chat::messageIn(QString message){
	if(message.length() == 0 || message == "" )
		message = "  ";

	if (chatSettings->chat_raise_window == true){
		show();
		raise();
	}

	// render the incoming message in the chatwindow
	if (chatSettings->chat_ignore_buddyhtml == true)
		updateHTML(parseAndProcessALL(message), buddyName, "#800000" );
	else
		updateHTML(message, buddyName, "#800000" );

	// make a sound
	if (( soundSettings_chat_sound == true ) &&
			( soundSettings_chat_sound_beepReceive == true)){
		beep();
	}
}

/***************************************************************************
 * update (render) the HTML in chatwindow widget
 ***************************************************************************/
void Chat::updateHTML(QString message, QString sender, QString color){
	stripTags(message);

	// Add img tags for smilies
	if( chatSettings->chat_show_smilies )
		message = updateHTMLSmilyPic(message); 

	// render at least a space
	if (message.length() == 1 || message.length() == 0 || message == "")
		message += " ";

	// Get Timestamp info
	QString CurrentTime = QTime::currentTime().toString().data();


	// Get the font tag...
	QString fontTag;
	if (sender == myName)
		fontTag = fontTagHTML(fontSize);
	else
		fontTag = fontTagHTML(3);
		
	// Wrap it around the message...
	if(!fontTag.isEmpty()){
		 fontTag = "<FONT" + fontTag + ">";
		 message = fontTag + message + "</FONT>";
	}


	/***** Get the name and : ****/
	QString TimeMessage = QString("<B><font color=\"") +
					  color + QString("\">") + CurrentTime + " " + sender + ": </font></B>"  + message;
	QString Message = QString("<B><font color=\"") +
	  color + QString("\">") + " " + sender + ": </font></B>"  + message;

	// Add to the logs.
	if (chatSettings_chat_log == true){
		logStream << parseAndProcessALL(TimeMessage) << "\n";
		logFile.flush();
	}


	Message.replace( QRegExp("\n"), "<br/>" );
	TimeMessage.replace( QRegExp("\n"), "<br/>" );

Message = QString("<table bgcolor=\"#ffffff\" border=0 cellpadding=0 cellspacing=0><tr><td>") + Message + "</td></tr></table><br/>";
TimeMessage = QString("<table bgcolor=\"#ffffff\" border=0 cellpadding=0 cellspacing=0><tr><td>") + TimeMessage + "</td></tr></table><br/>";

	// now add the message to the buffer
	conversationBuffer += Message;
	conversationTimeBuffer += TimeMessage;

	if(chatSettings_chat_show_time == true){
		if (postNumber == 0)
			chatWindow->setText(TimeMessage);
		else
			chatWindow->append(TimeMessage);
	}
	else{
		// NO TIME
		if (postNumber == 0)
			chatWindow->setText(Message);
		else
			chatWindow->append(Message);
	}

	// Incriment the post numbers
	postNumber++;
	if (postNumber == 1){
		if (( soundSettings_chat_sound == true ) &&
			(soundSettings->chat_sound_beepInitial == true)){
			beep();
		}
	}

	// scroll to the new text
	chatWindow->ensureVisible(0, chatWindow->contentsHeight());
	// This should not be a hard coded value like this!!!
	chatWindow->repaintContents(0,0, 10000, 29000, true);

	// if conversationLength is getting too long, cut the top out
	if (chatSettings->chat_line_limit == true)
		if(postNumber > chatSettings->chat_line_limit_number){
			int spos = conversationBuffer.find("<B>");
			conversationBuffer.remove(spos, 4);
			spos = conversationBuffer.find("<B>");
			conversationBuffer.remove(0, spos);
			postNumber--;
			chatWindow->setText( conversationBuffer, chatWindow->context());
		}


}

/***************************************************************************
 * convert smilies to images and add img tag
 ***************************************************************************/
QString Chat::updateHTMLSmilyPic(QString message){

#define FACE(x) \
	QString("<img width=\"19\" height=\"19\" src=\"" + QString(DATADIR) + "/icons/face_" + QString(x) + ".png\">")

	/* This first if is to check if a smiley is contained in the message.  It
	 * allows us to skip the big mess of message.replace's if there are no
	 * smilies, resulting in (a little) faster performance.  All the smilies
	 * contain either a '-' (nose) or a ':' (eyes) or a ')' (mouth)
	*/
	if((message.contains('-') != 0) ||    //find a nose
		(message.contains(')') != 0) ||     //or a mouth
		(message.contains(':') != 0)) {     //or some eyes.
		
		// tokenize odd combos that might be mistaken for smilies
		// see untokenize below
		message.replace(QRegExp("&gt;)"), "&gt;%29"); // when user types >)

		message.replace(QRegExp("=-[Oo]"), FACE("scream") );
		message.replace(QRegExp("&gt;:[Oo]"), FACE("yell") );
		message.replace(QRegExp("[Oo]:-)"), FACE("angel") );
		message.replace(QRegExp("C:-)"), FACE("luke") );

		message.replace(QRegExp(":-?)"), FACE("smile") ); // :-) :)

		message.replace(QRegExp(":-?("), FACE("sad") ); // :-( :(
		message.replace(QRegExp(";-?)"), FACE("wink") ); // ;-) ;)
		message.replace(QRegExp(":-?P"), FACE("tongue") ); // :-P :P

		message.replace(QRegExp(":-\\*"), FACE("kiss") );
		message.replace(QRegExp("8-)"), FACE("glasses") );

		message.replace(QRegExp("[Oo]-)"), FACE("oneeye") );

		message.replace(QRegExp(":-\\$"), FACE("moneymouth") );
		message.replace(QRegExp(":-!"), FACE("burp") );
		message.replace(QRegExp(":-\\["), FACE("embarrassed") );

		message.replace(QRegExp(":-\\"), FACE("think") );
		message.replace(QRegExp(":-/"), FACE("think") );
		message.replace(QRegExp(":'("), FACE("cry") );
		message.replace(QRegExp(":-X"), FACE("crossedlips") );
		message.replace(QRegExp(":-?D"), FACE("bigsmile") ); // :-D :D

		// UNtokenize odd combos that might be mistaken for smilies
		message.replace(QRegExp("&gt;%29"), "&gt;)"); // when user types >)
	}
	return message;
#undef FACE
}

/***************************************************************************
 * Parses the message
 ***************************************************************************/
#define SKIP_TO_GT {while(newstr[++i] != '>' && i < newstr.length());}

#define KILL_LT { newstr.replace(i, 1, "&lt;"); i += 3; }

#define SKIP_TO_GT_CHECK_QUOTE {int quotecount = 0;\
					while(newstr[++i] != '>' && i < newstr.length()){\
						if(newstr[i] == '"') quotecount++;}\
					if(quotecount % 2 == 1){\
						newstr.replace(i, 1, "\">");\
						i++;}}

QString Chat::parseAndProcessALL(QString &newstr){
	QString returnVal = "";
	unsigned int beg = 0;
	int openTag = 1;
	while (openTag == 1)
	for(unsigned i = 0; i <= newstr.length(); i++){
		if (i == 0)
			openTag = 0;
		if(newstr[i] == '<'){
			beg = i;
			SKIP_TO_GT
			unsigned int dif = i-beg;
			newstr.remove(beg,dif+1);
			openTag = 1;
			returnVal = "";
			i = newstr.length();
		}
		else returnVal += newstr[i];
	}
	return returnVal;
}

/***************************************************************************
 * Parses the message
 ***************************************************************************/
void Chat::parseAndProcess(QString &newstr){
	//make this more modular to allow for easier additions in the future.
	newstr = parseForHTML(newstr);
	//TODO: IF selected in options dialog
	//newstr = parseForSpelling(newstr);
	//newstr = parseForAutoReplace(newstr);
}

QString Chat::parseForHTML(QString &newstr){
	bool bodyUsed = false;
	bool divUsed = false;
	bool pUsed = false;

	for(unsigned i = 0; i < newstr.length(); i++)
	{
		if(newstr[i] == '>')
		{
			newstr.replace(i, 1, "&gt;");
			i += 3;
		}

		else if(newstr[i] == '<')
		{
			if(newstr.mid(i, 3).upper() == "<A ") SKIP_TO_GT_CHECK_QUOTE
			else if(newstr.mid(i, 3).upper() == "<B>") i += 2;
			else if(newstr.mid(i, 5).upper() == "<BIG>") i += 4;
			else if(newstr.mid(i, 4).upper() == "<BR>") i += 3;
			else if(newstr.mid(i, 6).upper() == "<CODE>") i += 5;
			else if(newstr.mid(i, 4).upper() == "<EM>") i += 3;
			else if(newstr.mid(i, 5).upper() == "<FONT") SKIP_TO_GT_CHECK_QUOTE
			else if(newstr.mid(i, 3).upper() == "<I>") i += 2;
			else if(newstr.mid(i, 7).upper() == "<SMALL>") i += 6;
			else if(newstr.mid(i, 4).upper() == "<TT>") i += 3;
			else if(newstr.mid(i, 3).upper() == "<U>") i += 2;
			else if(!divUsed && newstr.mid(i, 4).upper() == "<DIV")
		{
			divUsed = true;
			SKIP_TO_GT_CHECK_QUOTE
		}
		else if(!pUsed && newstr.mid(i, 2).upper() == "<P")
		{
			pUsed = true;
			SKIP_TO_GT_CHECK_QUOTE
		}
		else if(!bodyUsed && newstr.mid(i, 5).upper() == "<BODY")
		{
			bodyUsed = true;
			SKIP_TO_GT_CHECK_QUOTE
		}
		else if(newstr.mid(i, 7).upper() == "</HTML>") KILL_LT
		else if(newstr.mid(i, 7).upper() == "</DIV>") KILL_LT
		else if(newstr.mid(i, 7).upper() == "</BODY>") KILL_LT
		else if(newstr.mid(i, 2).upper() == "</") SKIP_TO_GT
		else KILL_LT;
		}
	}
	return QString::null;
}

/***************************************************************************
 * Strip the tags from string
 ***************************************************************************/
void Chat::stripTags( QString &text){
// 1) cut out <html> and </html> tags.
  int bodyPos = text.find( QRegExp("<html>", false) );
  while( bodyPos != -1 ){
    text.replace(bodyPos, 6, "");
    bodyPos = text.find( QRegExp("<html>", false) );
  }
  bodyPos = text.find( QRegExp("</html>", false) );
  while( bodyPos != -1 ){
    text.replace(bodyPos, 7, "");
    bodyPos = text.find( QRegExp("</html>", false) );
  }

  // 2a) cut <p> tags, replace </p> with <br>
  text.replace( QRegExp("<p>", false), "" );
  text.replace( QRegExp("</p>", false), "<br>" );
  // 2b) cut <pre> and </pre> tags
  text.replace( QRegExp("<pre>", false), "" );
  text.replace( QRegExp("</pre>", false), "" );

  // 3) replace <body> with <div>, inserting a STYLE= value to apply text and
  text.replace( QRegExp("<body>", false), "" );
  text.replace( QRegExp("</body>", false), "" );

  // 4) make sure the allowed tags are balanced, inserting a </tag>
  //    at the end of the message to balance if needed.
  #define CLOSE_TAG(tag) \
    if( text.contains("<" tag ">", false) > text.contains("</" tag ">", false) ) \
      text += ("</" tag ">");
  CLOSE_TAG("a");
  CLOSE_TAG("b");
  CLOSE_TAG("big");
  CLOSE_TAG("code");
  CLOSE_TAG("em");
  CLOSE_TAG("font");
  CLOSE_TAG("i");
  CLOSE_TAG("small");
  CLOSE_TAG("tt");
  CLOSE_TAG("u");
  
  #undef CLOSE_TAG

	// 4b) check allowed tags, make sure quotes are balanced within them
	// 5) replace any "<" or ">" that's not part of an allowed tag with &lt; or &gt;

        // 6) Weirdness from winAIM.  Sometimes <B> or <I> is immediately followed by </FONT> and this
        //    breaks Textview...which is weird itself in some ways :-)
        //    note: winaim always seems to send combos as <B><I>
        text.replace( QRegExp("<B></FONT>"), "</FONT><B>");
        text.replace( QRegExp("<I></FONT>"), "</FONT><I>");
        text.replace( QRegExp("<B><I></FONT>"), "</FONT><B><I>");
        // 6a) Weirdness in TextView. Doesn't like </B></FONT> etc.
        //     apparently </FONT> is enough and it breaks otherwise.
        text.replace( QRegExp("</B></FONT>"), "</FONT>");
        text.replace( QRegExp("</I></FONT>"), "</FONT>");
        text.replace( QRegExp("</B></I></FONT>"), "</FONT>");

  parseForHTML(text);
}

/***************************************************************************
 * Opens a log file
 ***************************************************************************/
void Chat::openLogFile(void){
	if(!chatSettings_chat_log)
		chatSettings_chat_log = true;

	QString normedName = buddyName;

	// Each chat gets it's own file.
	QString endingTag = ".txt";
	QString date = ((QDate::currentDate ()).toString());
	date.replace( QRegExp(" "), "" );
	date = date.mid(3,date.length()-3);
	QString month_day = date.left(date.length()-4);
	QString year = date.right(4);
	date = month_day + "-" + year;

	//QString logFolder = QDir::homeDirPath() + "/." + QString(APP_LOWERCASE_NAME) + "/logs/" + normedName + "/";
	QString logFolder = "/home/root/Documents/";
	QString fileNamePath = logFolder + normedName + "-" + date + endingTag;

	logFile.setName(fileNamePath);
	int logNumber = 2;
	QString logNumberString;
	while (logFile.exists() == true){
		logNumberString.setNum(logNumber);
		fileNamePath = logFolder + normedName + "-" + date + "-" + logNumberString + endingTag;
		logFile.setName(fileNamePath);
		logNumber++;
	}

	if (logFile.open( IO_WriteOnly | IO_Append ) == false){
        	QMessageBox::critical(0, APP_NAME, QString("Unable to open ") + fileNamePath + " for appending.", "OK");
			chatSettings_chat_log = false;
			fileMenu->setItemChecked(ID_FILE_LOG, false );
			if (toolBarSettings_chat_log == true && chatLogButton != NULL)
				chatLogButton->setDown ( false );
			return;
	} else{
			logStream.setDevice(&logFile);
			logStream << "Conversations with " << buddyName << "\n";
			logStream << " at " << QDate::currentDate().toString().data() << "\n";
			if (toolBarSettings_chat_log == true && chatLogButton != NULL)
				chatLogButton->setDown ( true );
	}


	if ( chatSettings_chat_log == true)
	  logStream << parseAndProcessALL(conversationTimeBuffer);
	
	// Flush everything we have so far.
	logFile.flush();
}

/***************************************************************************
 * Sends the message wehn enter is allowed.
 * The return key itself will add a line so it is removed
 ***************************************************************************/
void Chat::enterKeyPressed(){
  if(chatSettings->chat_return_send == true){
    if (currentMessageOut.length() > 2000){
      QMessageBox::information(0, APP_NAME, "Your message has more than 2000 letters.  It is too long and can not be sent, make it shorter.", "OK");
      return;
    }
    // Repace the text with what was just there before the return. :)
    inputWindow->setText(currentMessageOut);
    lastMessage = currentMessageOut;
    slotBuddySendMessage();
    currentMessageOut = QString("");
  }
  // Else just let a return be placed
}

/**************************************************************************
	* return HTML fontTag
	*************************************************************************/
QString Chat::fontTagHTML(int size){
	QString fontTag;
	QString fontSizeTag;

		// Add font current size
		if (fontSize != 3) {
	fontSizeTag.setNum(size);
		}

		// Build tag
		if(!fontSizeTag.isEmpty()){
	fontTag += " SIZE=\"" + fontSizeTag + "\"";
		}

	return fontTag;
}

/**************************************************************************
	* return message with HTML fixed for winAim smilie rendering
	*************************************************************************/
QString Chat::fontFixSmilies(QString message, QString fontTag){
	//If we dont have an already open fontTag, then we dont need to send the
	//close font tag before enclosing the smiley in its own font tags.
	QString preTag = fontTag.isEmpty() ? "" : "</FONT>";

#define FONTTAG(x) \
	QString(preTag + "<FONT SIZE=\"3\">" + QString(x) + "</FONT>" + fontTag)

	 // plug in fix so that aol users get smiles in font size 3 (so that it matches what we see)
	 // please do not remove the extra fontag--winaim won't render reliably w/o it ;-)
	 if( (message.contains('-') != 0) ||     //find a nose
		 (message.contains(')') != 0) ||       //or a mouth
		 (message.contains(':') != 0) ){       //or some eyes.

				message.replace(QRegExp("=-[Oo]"), FONTTAG("=-O") );
				message.replace(QRegExp("&gt;:[Oo]"), FONTTAG(">:O") );
				message.replace(QRegExp("[Oo]:-)"), FONTTAG("O:-)") );
				message.replace(QRegExp("C:-)"), FONTTAG("C:-)") );

				// the "not" character gets counted in replace function..so it eats the one b4
				// message.replace(QRegExp("[^COo]:-?)"), FONTTAG(":-)") ); // :-) :)

				// wrap FONTTAG around smily w/o messing up the variations...
				int i = 0;
				while((i = message.find( QRegExp("[^COo]:-)"), i )) > 0)  {
					message.replace(i+1, 3, FONTTAG(":-)") );
					i = i + 29; // min length of FONTAG string macro
				}
				i = 0;
				while((i = message.find( QRegExp("[^COo]:)"), i )) > 0)  {
					message.replace(i+1, 2, FONTTAG(":)") );
					i = i + 29; // min length of FONTAG string macro
				}
				message.replace(QRegExp(":-?("), FONTTAG(":-(") ); // :-( :(
				message.replace(QRegExp(";-?)"), FONTTAG(";-)") ); // ;-) ;)
				message.replace(QRegExp(":-?P"), FONTTAG(":-P") ); // :-P :P

				message.replace(QRegExp(":-\\*"), FONTTAG(":-*") );
				message.replace(QRegExp("8-)"), FONTTAG("8-)") );
			
				message.replace(QRegExp("[Oo]-)"), FONTTAG("O-)") );
			
				message.replace(QRegExp(":-\\$"), FONTTAG(":-$") );
				message.replace(QRegExp(":-!"), FONTTAG(":-!") );
				message.replace(QRegExp(":-\\["), FONTTAG(":-[") );
			
				message.replace(QRegExp(":-\\"), FONTTAG(":-/") );
				message.replace(QRegExp(":-/"), FONTTAG(":-/") );
				message.replace(QRegExp(":'("), FONTTAG(":'(") );
				message.replace(QRegExp(":-X"), FONTTAG(":-X") );
				message.replace(QRegExp(":-?D"), FONTTAG(":-D") ); // :-D :D
	}

	return message;
#undef FONTTAG
}

/******************************************************************************
 * Sets if the buddy is on the buddy list of not.
 ******************************************************************************/
void Chat::setIfIsBuddy(QString name, bool isABuddy){
  if (name != buddyName)
    return;
  isBuddy = isABuddy;
  initToolBar();
  initMenuBar();
}


/******************************************************************************
 * Remove the buddy from the current list
 * Pass to main window for conferm/action
 ******************************************************************************/
void Chat::slotBuddyAddRemove(){
  if (isBuddy == true){
    emit( removeBuddy(buddyName) );
  }
  else{
    emit( addBuddy(buddyName) );
  }
}

/***************************************************************************
 * Constructor
 ***************************************************************************/
KaimTextBrowser::KaimTextBrowser(QWidget *parent, const char *name ) : QTextBrowser(parent,name){
}

/***************************************************************************
 * When a url is clicked this is done instead of the normal.
 ***************************************************************************/
void KaimTextBrowser::setSource(const QString& name){
  emit gotoURL(name);
}

/******************************************************************************
 * update buddyName
 ******************************************************************************/
void Chat::updateBuddyName(QString buddy){
  buddyName = buddy;
}

// chat.cpp

