/***************************************************************************
					                          chat.h
					                   -------------------
		begin                : Wednesday March 28 2000
		copyright            : (C) 2001 by Benjamin Meyer
		email                : ben@meyerhome.net
 ***************************************************************************/

/***************************************************************************
 * This is the base class for a chat. 
 ***************************************************************************/

#ifndef CHAT_H
#define CHAT_H

#include <qmainwindow.h>
#include <qtextbrowser.h>
#include <qmultilineedit.h>
#include "browser.h"
#include "setup.h"

class QToolButton;
class QToolBar;
class QLabel;
class QTextStream;
class QFile;
class QColor;
class QPopupMenu;
class QVBoxLayout; 
	
/**
 * @author Benjamin Meyer
  */

class KaimTextBrowser : public QTextBrowser {
  Q_OBJECT 
  public:
    KaimTextBrowser(QWidget *parent=0, const char *name=0 );
  
  signals:
    void gotoURL(QString url);	

  protected:
    void setSource(const QString& name);

};

class Chat : public QMainWindow  {

	Q_OBJECT

  signals:
    void removeBuddy(QString name);
    void blockBuddy(QString name);
    void warningOut(QString userName);
	void messageOut(QString buddy, QString Message, int room);
	void addBuddy(QString name);
	void getInfo(QString Name);
	void quiting(QString name, int room);
	void saveToolbarSettings();
		
	public:
		Chat(QString userName, QString buddysName, bool buddy, QWidget *parent=0);
		~Chat();
		void buddySignedOff();
		void buddySignedOn();
		void awayMessage(QString message);
		bool sentAwayMessageToBuddy;
		Browser *browser;
		QString name;
		QPopupMenu *windows;
		void addWindowsMenu();
		void updateBuddyName(QString buddy);

	protected:
		void closeEvent(QCloseEvent *);
		// Internal Settings
		QString myName;
		QString buddyName;
		QVBoxLayout* Form1Layout;
		QWidget *bigOne;

		// The widgets in the tool bar
		QToolButton *sendMessageButton;
 
		QToolButton *chatClearButton;
		QToolButton *chatTimeButton;
		QToolButton *chatLogButton;
		QToolButton *chatSoundMuteButton;
 
		QToolButton *textBoldButton;
		QToolButton *textItalicButton;
		QToolButton *textUnderLineButton;
		QToolButton *textStrikeButton;
 
		QToolButton *textUpFontButton;
		QToolButton *textNormFontButton;
		QToolButton *textDownFontButton;
		QToolButton *smileyButton;
	
		// The Menu Bars
		QPopupMenu *fileMenu;
		QPopupMenu *editMenu;
		QPopupMenu *viewMenu;
		QPopupMenu *insertMenu;
		QPopupMenu *insertFaceMenu;
		QPopupMenu *buddyMenu;

	 	QToolBar *fileToolbar;
		QToolBar *textToolbar;

		// The init function for the widget
		virtual void initToolBar();
		virtual void initMenuBar();
		virtual void initChatWindow();
	
		//QLineEdit *spliter;

		//The following are for delayed local settings creation.
		chat_ToolBar_Settings *toolBarSettings;
		chat_Settings *chatSettings;
		chat_Sound_Settings *soundSettings;

		bool chatSettings_chat_log;
		bool toolBarSettings_chat_log;
		bool soundSettings_chat_sound;
		bool soundSettings_chat_sound_beepReceive;
		bool soundSettings_chat_sound_beepSend;
		bool chatSettings_chat_show_time;

	private:
		QString currentMessageOut;
		bool canCopy;
		int fontSize;
		int normalFontSize;
		
		QString lastMessage;

		// The Log Data's
		QFile logFile;
		QTextStream logStream;

		// The two buffers, one with time, one without.
		QString conversationBuffer;
		QString conversationTimeBuffer;
		
		// Normal function calls in the app
		void updateHTML(QString message, QString sender, QString color);
		QString parseAndProcessALL(QString &newstr);
		void parseAndProcess(QString &newstr);
		QString parseForHTML(QString &newstr);
	
		void stripTags(QString &oldstr);
		void openLogFile(void);
		QString fontTagHTML(int size);
		QString fontFixSmilies(QString message, QString fontTag);
		QString updateHTMLSmilyPic(QString message);
 
		// The Chat window
		KaimTextBrowser *chatWindow;
		
		// used to get around the 0/1 issue on chatWindow input
		int postNumber;
		QMultiLineEdit *inputWindow;
    QToolButton *userRemoveButton;
    QToolButton *userWarnButton;
    QToolButton *userBlockButton;
    QToolButton *userInfoButton;
    bool isBuddy;

	public slots:
		void messageIn(QString message);
		
		void updateChatSettings();
		void updateChatSoundSettings();
    void setIfIsBuddy(QString name, bool isABuddy);
    void updateChatToolBarSettings();
	
  private slots:
    inline void slotBuddyBlock() { emit( blockBuddy(buddyName) ); };
    inline void slotBuddyWarn() { emit( warningOut(buddyName) ); };
    void slotBuddyAddRemove();
    inline void slotBuddyInfo() { emit( getInfo( buddyName ) ); };

    void enterKeyPressed();
    void applytoInputWindow();
    inline void inputTextChanged() { currentMessageOut = inputWindow->text();};
    inline void copyChanged(bool copy) { canCopy = copy; };
    inline void urlClicked(QString url) { browser->openURL(url); };
		
    void slotChatLog();
    void slotChatClear();
    void slotChatClose();
    void slotChatSound();

    void slotEditTurnOnOffUndo(bool);
    void slotEditTurnOnOffRedo(bool);
    void slotEditCopy();
    void slotEditSelectAll();

    void slotViewChatToolBar();
    void slotViewTextToolBar();
    void slotViewTimeStamp();

    void slotInsertFace(int);
    void slotInsertTags(QString open, QString close);
    void slotInsertBold();
    void slotInsertItalic();
    void slotInsertUnderline();
		void slotInsertShrinkFont();
		void slotInsertNormalFont();
		void slotInsertIncreaseFont();
		void slotInsertLastMessage();
	
		void slotBuddySendMessage();

};

#endif

// chat.h


