/******************************************************************************
                                  kinkatta.cpp
                             -------------------
    begin                : Sat May 20 2000
    copyright            : (C) 2000 by Benjamin Meyer
    email                : benjamin@csh.rit.edu
 ******************************************************************************/

/******************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify it
 *   under the terms of the GNU General Public License as published by the
 *   Free Software Foundation; either version 2 of the License, or (at your
 *   option) any later version.
 *
 ******************************************************************************/

/******************************************************************************
 * Main QObject for application.  All other windows are kept as children of
 * this class.  They are created and distroyed when needed.  Data pointer are
 * passed to the window that are needed.  These include the connection and
 * the settings.
 ******************************************************************************/

#include "kinkatta.h"
#include <qapplication.h>
#include "qmessagebox.h"
#include "application.h"
#include "aim.h"
#include <qtimer.h>

#define SOUND_MUTE_ON_LOGIN_TIME 12


/******************************************************************************
 * Constructor
 ******************************************************************************/
Kaim::Kaim(QWidget* parent, const char *name):QMainWindow(parent, name) {
  windows = new QPopupMenu(this, "Menu");
  loggingOnError = false;
  quiting = false;
  manualLogOff = false;
  progress = NULL;
  loginIn = false;
  optionsDialog = NULL; 
  browser = new Browser();
  this->setIcon(Icon("kinkatta_small"));
  connect(this, SIGNAL(settingsChanged()), this, SLOT(SaveAimSettings()));
  
  // Get settings from file and set them to the current aimSettings
  setup::instance()->read();
  i_aimSettings = setup::settings();
  browser->otherBrowser = i_aimSettings->main_browser;
  // init the connection
  connection = new TAim(this, "Connection");
  connect(connection, SIGNAL(statusChanged(int)), this, SLOT(watchConnection(int)));
  connect(connection, SIGNAL( displayError(QString)), this, SLOT( displayError(QString)));
  connect(connection, SIGNAL(nick(QString)), this, SLOT(setNick(QString)));
  connect(connection, SIGNAL(recievedServerConfig()), this, SLOT(importServerConfig()));	
  connect( connection, SIGNAL(initProgress(int, int, QString)), this, SLOT(initProgress(int, int, QString)) );
  connect( connection, SIGNAL(updateProgress(int, QString)), this, SLOT(updateProgress(int, QString)) );
  connect( connection, SIGNAL(endProgress()), this, SLOT(endProgress()) );
	
  buddyWindow = new BuddyListWindow(this, "buddyDialog", connection);
  buddyWindow->browser = browser;
  buddyWindow->windows = windows;
  buddyWindow->initMenuBar();
  buddyWindow->fillUpWindows();

  connect(buddyWindow, SIGNAL(quit()), this, SLOT(Quit()));
  connect(buddyWindow, SIGNAL(logOff()), this, SLOT(Logoff()));
  connect(buddyWindow, SIGNAL(showOptions()), this, SLOT(BringUpSetup()));
  connect(buddyWindow, SIGNAL(showAwayOptions()), this, SLOT(showAwayOptions()));
  connect(buddyWindow, SIGNAL(saveSettings()), this, SLOT(SaveAimSettings()));
  connect(buddyWindow, SIGNAL(set_window(int,int)), this, SLOT(resize(int,int))); 
  connect(this, SIGNAL(settingsChanged()), buddyWindow, SLOT(settingsChanged()));
  buddyWindow->hide();
 
  loginDialog = NULL;
  /*
  loginDialog = new Login(this, "LoginDialog");

  connect(loginDialog, SIGNAL(setupClicked()), this, SLOT(BringUpSetup()));
  connect(loginDialog, SIGNAL(signOnClicked()), this, SLOT(TryToConnect()));
  connect(loginDialog, SIGNAL(destroyed()), this, SLOT(Quit()));
  connect(this, SIGNAL(settingsChanged()), loginDialog, SLOT(updateSettings()));
  loginDialog->browser = browser;
  loginDialog->hide();
  */

  // Set state to Disconected();
  Disconected();
  this->setCaption(APP_NAME);
  showMaximized();
}

/******************************************************************************
 * Set the proper nick spacing etc.
 ******************************************************************************/
void Kaim::setNick(QString name){
  i_aimSettings->login_name = name;
}	

/******************************************************************************
 * Network code had an error, display
 ******************************************************************************/
void Kaim::displayError(QString error){
  loggingOnError = true;
  if (manualLogOff == true){
    if (error != "Disconnected.")
      QMessageBox::critical(0, APP_NAME " - Error",error, "Ok");
    manualLogOff = false;
	}
  else
	  QMessageBox::critical(0, APP_NAME " - Error",error, "Ok");
}

/******************************************************************************
 * Kaim has been disconnected, bring up login screen
 ******************************************************************************/
void Kaim::Disconected(){
  if (connection->getStatus() != TAIM_OFFLINE)
    return;
 
  if(buddyWindow->awayRightNow == true)
      buddyWindow->away_BackForReal();
  
  buddyWindow->hide();
  buddyWindow->clearPostConnections();
  if(loginDialog == NULL){
    loginDialog = new Login(this, "LoginDialog");
    connect(loginDialog, SIGNAL(setupClicked()), this, SLOT(BringUpSetup()));
    connect(loginDialog, SIGNAL(signOnClicked()), this, SLOT(TryToConnect()));
    connect(loginDialog, SIGNAL(destroyed()), this, SLOT(Quit()));
    connect(this, SIGNAL(settingsChanged()), loginDialog, SLOT(updateSettings()));
    loginDialog->browser = browser;
  }
  loginDialog->show();

  // Bring up the Login Window
  loginDialog->setFocus();
  this->setCentralWidget(loginDialog);
  loginDialog->updateSettings();
  if (loggingOnError == false)
    if (i_aimSettings->login_enable_auto == true)
      loginDialog->SignOnClicked(); 
  showMaximized(); 
}

/******************************************************************************
 * The Login Screen wants to try to log in.
 ******************************************************************************/
void Kaim::TryToConnect(){
  if (connection->getStatus() != TAIM_OFFLINE)
    return;
  loginIn = true;
 
  loggingOnError = false;

  // Bring up the SignOn Window
	//save the options again, in case Options was pressed under a different user
	setup::instance()->readProfile( setup::settings()->login_name );
	
  TBuddyList buddy;
  TBuddyList permit;
  TBuddyList deny;
  int permitStatus;
  
  
  setup::instance()->parseBuddyList(setup::settings()->login_name, 
																&buddy, &permit, &deny, &permitStatus);

  connection->setBuddyList(buddy);
  connection->setPermitList(permit);
  connection->setDenyList(deny);
  connection->setPermissions(permitStatus);
  
  // dokeepAlive
  connection->doKeepAlive(true);
  // Idle
  connection->doIdleTime(true);
  // SetCustom server
  connection->useCustomServer(true);
  connection->setCustomServer(setup::settings()->login_server_toc,setup::settings()->login_port_toc.toInt(),setup::settings()->login_server_login,setup::settings()->login_port_login.toInt());
  // Enable proxy
  if (setup::settings()->proxy == true){
    connection->useSocksServer(true);
    if( setup::settings()->proxy_server.isEmpty()){
      QMessageBox::critical(0, APP_NAME " - Error","No Proxy server selected.", "Ok");
	return;
    }
    connection->setSocksProxy(setup::settings()->proxy_server,setup::settings()->proxy_port.toInt());
  }
  
  // Set UserName
  connection->setUserName( setup::settings()->login_name );
  // set Pass
  connection->setPassword( setup::settings()->login_password );
  // Set UserInfo
  connection->setUserInfo( setup::instance()->readProfile(i_aimSettings->login_name) );
  // setStatus(taim_online)
  connection->setStatus(TAIM_ONLINE);
}

/******************************************************************************
 * Kaim is connected to a toc server, Bring up buddylist
 ******************************************************************************/
void Kaim::Connected(){
  if (connection->getStatus() != TAIM_ONLINE){
    return;
  }
	
  // If this a re-signon with a different user, close the previous users
  // chatwindows.  If this is a re-signon with the same user, leave existing
  // chatwindows open.
  if( lastLoginName != setup::settings()->login_name ){
    buddyWindow->closeExistingChatwindows();
  }
  lastLoginName = setup::settings()->login_name;
  
  if (connection->getStatus() != TAIM_OFFLINE){
    // If this a re-signon with a different user, close the previous users
    // chatwindows.  If this is a re-signon with the same user, leave existing
    // chatwindows open.
    if( lastLoginName != setup::settings()->login_name ){
      buddyWindow->closeExistingChatwindows();
    }
    lastLoginName = setup::settings()->login_name;
 
    buddyWindow->show();
    this->setCentralWidget(buddyWindow);
    if(loginDialog->close())
      loginDialog = NULL;

    qApp->processEvents();
    QTimer::singleShot( SOUND_MUTE_ON_LOGIN_TIME*1000, buddyWindow, SLOT(endMutePeriod()));
    //buddyWindow->startMutePeriod();
    buddyWindow->loadList();
    if(buddyWindow->awayRightNow == true)
      buddyWindow->away_BackForReal();
    buddyWindow->initOptions();
    SaveAimSettings();
  }
}

/******************************************************************************
 * Someone wants the setup window, bring it up.  Saves when exiting.
 * The options window internally handles if changes are made.
 ******************************************************************************/
void Kaim::BringUpSetup(){
  // Option Box
  if ( optionsDialog == NULL ) {
    optionsDialog = new options(0, "options dialog", connection);
    optionsDialog->setIcon(Icon("kinkatta_small"));
    connect(optionsDialog, SIGNAL(dataUpdated()), this, SIGNAL(settingsChanged()));
    connect(optionsDialog, SIGNAL(setDefault()), this, SLOT(setDefaultSettings()));
    optionsDialog->updateData();
    optionsDialog->windows = windows;
    optionsDialog->addWindowMenu();
    optionsDialog->show();

  } else {
  /* If we get here, then the optionsDialog != NULL, which means it has
   * already been allocated/connected up, so we can just updateData, and
   * show() it.
   */
    //optionsDialog->updateData();
    optionsDialog->show();
    optionsDialog->raise();
  }
}

void Kaim::showAwayOptions(){
  BringUpSetup();
  optionsDialog->slotShowAway();
}

/******************************************************************************
 * Catches offline signals and acts
 ******************************************************************************/
void Kaim::watchConnection(int n){
  if ( n == TAIM_OFFLINE){
    if (quiting == false && loginIn == false)
      Disconected();
  }
  else
    if (loginIn == false)
      Connected();
}

/******************************************************************************
 * Main window requests to log off.  Go offline and bring up login window.
 ******************************************************************************/
void Kaim::Logoff(){
  manualLogOff = true;
	connection->setStatus(TAIM_OFFLINE);
  Disconected();
	SaveAimSettings();
}

/******************************************************************************
 * Save the current settings to file.
 * Occurs when settings are changed (i.e. login and setup)
 ******************************************************************************/
void Kaim::SaveAimSettings(){
  //QString conf;
  setup::instance()->save();
  if (connection->getStatus() != TAIM_OFFLINE){
    setProfile();
    
    setup::instance()->saveBuddyList( setup::settings()->login_name,
			&(connection->buddyList()), &(connection->permitList()),
			&(connection->denyList()), connection->permissions() );
	}
}

/***************************************************************************
 * Reset the settings and the update the options
 ***************************************************************************/
void Kaim::setDefaultSettings(){
  setup::instance()->resetDefaults();
  optionsDialog->updateData();
}

/***************************************************************************
 * Sets the current profile on the server to what is in the settings. 
 ***************************************************************************/
void Kaim::setProfile(){
  connection->setUserInfo( setup::instance()->readProfile( setup::settings()->login_name) );
}

/***************************************************************************
 * Quits the application
 * Logs off and saves first
 ***************************************************************************/
void Kaim::Quit(){
  if (connection->getStatus() != TAIM_OFFLINE){
    quiting = true;
    manualLogOff = true;
    connection->setStatus(TAIM_OFFLINE);
  } 
  buddyWindow->closeExistingChatwindows();

  SaveAimSettings();
  close();  
}

void Kaim::initProgress(int max, int cur, QString msg)
{
	if(progress) delete progress;
	progress = new ProgressDialog(this, APP_NAME " - Login");
	progress->setCaption(APP_NAME " - Signon");
	progress->setIcon(Icon("kinkatta_small"));
	//progress->setTotalSteps(max);
	connect( progress, SIGNAL(cancelled()), connection, SLOT(cancelProgress()) );
	progress->setProgress(cur);
	progress->setLabelText(msg);
	progress->show();
}
void Kaim::updateProgress(int cur, QString msg)
{
	if(!progress) return;
	progress->setProgress(cur);
	progress->setLabelText(msg);
}
void Kaim::endProgress(void)
{
  if(!progress) return;
  delete progress;
  progress = 0;
  loginIn = false;
  if (connection->getStatus() != TAIM_OFFLINE)
    Connected();
}

void Kaim::importServerConfig(){
  TBuddyList slist = connection->serverBuddyList();
  TBuddyList list = connection->buddyList();
  list += slist;
  connection->setBuddyList(list);
  buddyWindow->loadList();
}

// kinkatta.cpp

