/******************************************************************************
                                 kinkatta.h
                             -------------------
    begin                : Sat May 20 2000
    copyright            : (C) 2000 by Benjamin Meyer
    email                : benjamin@csh.rit.edu
 ******************************************************************************/

/******************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify it
 *   under the terms of the GNU General Public License as published by the
 *   Free Software Foundation; either version 2 of the License, or (at your
 *   option) any later version.
 *
 ******************************************************************************/

/******************************************************************************
 * Main QObject for application.  All other windows are kept as children of
 * this class.  They are created and distroyed when needed.  Data pointer are
 * passed to the window that are needed.  These include the connection and
 * the settings.
 ******************************************************************************/

#ifndef KINKATTA_H
#define KINKATTA_H

#include <qmainwindow.h> 
#include "login.h"              // AIM Login Window
#include "buddylistwindow.h"    // AIM Buddy Lists Window
#include "taim.h"               // the toc protocal object
#include "setup.h"		          // AIM Settings
#include "tbuddylist.h"
#include "browser.h"
#include "options.h"
#include <qpixmap.h>
#include "progressdialog.h"
#include <qpopupmenu.h>

/**
  *@author Benjamin Meyer
  */

class Kaim : public QMainWindow  {
  Q_OBJECT

  signals:
    void settingsChanged();
	
  public:
    Kaim(QWidget* parent=0, const char *name=0);
		
  private:
    bool loginIn;
    Browser *browser;
    Login *loginDialog;
    BuddyListWindow *buddyWindow;
    options *optionsDialog;

    QString lastLoginName;
    QPopupMenu *windows;

    settings_t *i_aimSettings;
    TAim *connection;
    bool loggingOnError;
    bool quiting;
    bool manualLogOff;
    ProgressDialog *progress;

  private slots:
    void displayError(QString error);
    void Quit();
    void Connected();
    void TryToConnect();
    void Disconected();
    void BringUpSetup();
    void showAwayOptions();
    void SaveAimSettings();
    void Logoff();
    void watchConnection(int);
    void setProfile();
    void setDefaultSettings();
    void setNick(QString);	
    void importServerConfig();	
	
    // progress bar for aim
    void initProgress(int, int, QString);
    void updateProgress(int, QString);
    void endProgress(void);
 
};

#endif

// kinkatta.h

