/***************************************************************************
                               options.cpp
                             -------------------
    begin                : Tuesday Nov 14 2000
    copyright            : (C) 2000 Benjamin Meyer
    email                : benjamin@csh.rit.edu
 ***************************************************************************/

#include "options.h"

#include "chatoptionsimp.h"
#include "awayoptionsimp.h"
#include "mainwindowoptionsimp.h"
#include "profileoptions.h"
#include "soundoptionsimp.h"
#include "networkoptionsimp.h"
#include "permitdenyoptionsimp.h"

#include <qwidgetstack.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include <qmenubar.h>
#include <qpopupmenu.h>

/* 
 *  Constructs a options which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
options::options( QWidget* parent,  const char* name, TAim *connection, bool modal, WFlags fl )
    : QDialog( parent, name, modal, fl ){
  i_aimSettings = setup::settings();
  setCaption( tr( "Kinkatta Lite - Options" ) );

  Form1Layout = new QVBoxLayout( this ); 
  Form1Layout->setSpacing( 6 );
  Form1Layout->setMargin( 5 );

  bar = new QMenuBar(this, "Menu Bar");
  menu = new QPopupMenu(this, "popupmenu");
  bar->insertItem("&Options", menu);
  menu->insertItem("&Network", this, SLOT(slotShowNetwork()));
  menu->insertItem("&Chat", this, SLOT(slotShowChat()));
  menu->insertItem("&Main Window", this, SLOT(slotShowMain()));
  menu->insertItem("&Sound", this, SLOT(slotShowSound()));
  menu->insertItem("&Permit Deny", this, SLOT(slotShowDeny()));
  menu->insertItem("&Away", this, SLOT(slotShowAway()));
  menu->insertItem("&Profile", this, SLOT(slotShowProfile()));
  menu->insertSeparator();
  menu->insertItem("&Default", this, SIGNAL(setDefault()));
  menu->insertSeparator();
  menu->insertItem("Close &Window", this, SLOT(accept()));
  Form1Layout->setMenuBar( bar );

  configureWidget = new QWidgetStack( this, "configureWidget" );
  Form1Layout->addWidget( configureWidget );

  network = new NetworkOptionsImp( configureWidget, "NetworkPage");
  chat = new ChatOptionsImp(configureWidget, "ChatPage");
  main = new MainWindowOptionsImp(configureWidget, "MainPage");
  sound = new SoundOptionsImp( configureWidget, "SoundPage");
  deny = new PermitDenyOptionsImp( configureWidget, "DenyPage");
  deny->i_connection = connection;
  away = new AwayWindowOptionsImp( configureWidget, "AwayPage");
  profile = new ProfileOptions( configureWidget, "ProfilePage");
  
  configureWidget->addWidget(network, -1);
  configureWidget->addWidget(chat, -1);
  configureWidget->addWidget(main, -1);
  configureWidget->addWidget(sound, -1);
  configureWidget->addWidget(deny, -1);
  configureWidget->addWidget(away, -1);
  configureWidget->addWidget(profile, -1);
  configureWidget->raiseWidget(network);
  showMaximized();
}

void options::addWindowMenu(){
  bar->insertItem("|",1000);
  bar->setItemEnabled(1000,false);
  bar->insertItem("&Windows", windows);
}

void options::slotShowNetwork(){
  configureWidget->raiseWidget(network);
}

void options::slotShowChat(){
  configureWidget->raiseWidget(chat);
}

void options::slotShowMain(){
  configureWidget->raiseWidget(main);
}

void options::slotShowSound(){
  configureWidget->raiseWidget(sound);
}

void options::slotShowDeny(){
  configureWidget->raiseWidget(deny);
}

void options::slotShowAway(){
  configureWidget->raiseWidget(away);
}

void options::slotShowProfile(){
  configureWidget->raiseWidget(profile);
}

void options::accept(){
  applyClicked();
  QDialog::accept();
}

void options::cancelClicked(){
  QDialog::accept();
}

void options::applyClicked(){
  sound->getData();
  network->getData();
  deny->getData();
  away->getData();
  profile->getData();
  chat->getData();
  main->getData();
  emit (dataUpdated());
  emit (exiting());
}

void options::updateData(){
  i_aimSettings = setup::settings();

  sound->updateData();
  network->updateData();
  deny->updateData();
  away->updateData();
  profile->updateData();
  chat->updateData();
  main->updateData();
}

