#include "profileoptions.h"

#include <qframe.h>
#include <qlabel.h>
#include <qmultilineedit.h>
#include <qpushbutton.h>
#include "qwidget.h"
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>

/** 
 *  Constructs a ProfileOptions which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 */
ProfileOptions::ProfileOptions( QWidget* parent,  const char* name, WFlags fl )
    : QWidget( parent, name, fl )
{
    i_aimSettings = setup::settings();
    ProfileOptionsLayout = new QVBoxLayout( this ); 
    ProfileOptionsLayout->setSpacing( 6 );
    ProfileOptionsLayout->setMargin( 0 );

    title = new QLabel( this, "title" );
    title->setText( tr( "User Profile" ) );
    title->setAlignment( int( QLabel::AlignCenter ) );
    ProfileOptionsLayout->addWidget( title );

    Line1 = new QFrame( this, "Line1" );
    Line1->setFrameStyle( QFrame::HLine | QFrame::Sunken );
    ProfileOptionsLayout->addWidget( Line1 );

    profile_edit = new QMultiLineEdit( this, "profile_edit" );
    profile_edit->setMinimumSize( QSize( 0, 50 ) );
    profile_edit->setWordWrap( QMultiLineEdit::WidgetWidth );
    profile_edit->setWrapPolicy( QMultiLineEdit::Anywhere );
    profile_edit->setText( tr( "<P>Visit the Kinkatta website at: <A HREF=\"http://kinkatta.sourceforge.net/\">http://kinkatta.sourceforge.net/</A>. </P>" ) );
    ProfileOptionsLayout->addWidget( profile_edit );
    QSpacerItem* spacer = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );
    ProfileOptionsLayout->addItem( spacer );
}

/******************************************************************************
 * Copy the settings to the dialogs
 ******************************************************************************/
void ProfileOptions::updateData(){
  i_aimSettings = setup::settings();

  if(i_aimSettings->login_name.isEmpty()){
    this->setEnabled(FALSE);
    profile_edit->clear();
    title->setText( tr("User Profile") );
    return;
  } 
  else {
    title->setText( tr("User Profile for " + i_aimSettings->login_name) );
    setup::instance()->readProfile( i_aimSettings->login_name );
    this->setEnabled(TRUE);
  }
		
  profile_edit->setText( setup::instance()->readProfile(i_aimSettings->login_name) );
}

/******************************************************************************
 * Copy the settings to the dialogs
 ******************************************************************************/
void ProfileOptions::getData(){
  if(i_aimSettings->login_name.isEmpty())
    return;
		
  if(!i_aimSettings->login_name.isEmpty())
    setup::instance()->saveProfile(i_aimSettings->login_name, profile_edit->text());
}


