/**************************************************************************
                                setup.cpp
                             -------------------
    begin                : Thu Feb 17 2000
    copyright            : (C) 2000 by Benjamin Meyer
    email                : benjamin@csh.rit.edu
 ***************************************************************************/

#include "setup.h"
#include "application.h"
#include <qpe/config.h>
#include "taim.h"

//The default profile for new users, or users with no profile set yet.
#define DEFAULT_PROFILE "<P>Visit the " APP_NAME " website at: <A HREF=\"http://kinkatta.sourceforge.net/\">http://kinkatta.sourceforge.net/</A>.</P>"

setup *__instance = 0;

/***************************************************************************
 * Instance.  This is a static method that returns the current reference to
 * setup.  This is so we can avoid creating/deleting setup's all over the
 * place, and we dont need static methods to do simple things.
 ***************************************************************************/
setup *setup::instance(void){
  if( !__instance )
    __instance = new setup();
  return __instance;
}

/***************************************************************************
 * This is just a shortcut from having to write
 * setup::instance()->i_currentSettings every time
 ***************************************************************************/
settings_t *setup::settings(void){
  return setup::instance()->i_currentSettings;
}

/***************************************************************************
 * Constructor
 ***************************************************************************/
setup::setup(){
  i_currentSettings = new settings_t;
  
  // Set up basic defaults
  i_currentSettings->Version = "Version 1.00";
  i_currentSettings->ReleaseDate = "1/16/01";

  i_currentSettings->chatSettings = new chat_Settings;
  i_currentSettings->chatSoundSettings = new chat_Sound_Settings;
  i_currentSettings->chatToolBarSettings = new chat_ToolBar_Settings;
  
  i_currentSettings->mapAwayMessages = new QMap<QString, QString>; 
}

/***************************************************************************
 * Deconstructor
 ***************************************************************************/
setup::~setup(){
  delete i_currentSettings->chatSettings;
  delete i_currentSettings->chatSoundSettings;
  delete i_currentSettings->chatToolBarSettings;
  delete i_currentSettings->mapAwayMessages;

  delete i_currentSettings;
}

/***************************************************************************
 * Returns a list of all the registered users of kaim. By this, it means all
 * users that have a settings file under ~/.kaim/
 ***************************************************************************/
QStringList setup::registeredUsers(){
  Config prefs(APP_NAME);
  prefs.setGroup("savedUsers");
  return prefs.readListEntry("users",'/');
}

/***************************************************************************
 * Returns whether or not the passed user has used kaim before (ie, if he is
 * in the list returned by setup::registeredUsers())
 ***************************************************************************/
bool setup::isRegisteredUser(const QString &user){
  if( setup::registeredUsers().contains(user) != 0)
    return TRUE; //has used kaim, is registered
  else
    return FALSE; //has not used kaim, is not registered
}

/***************************************************************************
 * Save the settings to the file
 ***************************************************************************/
void setup::save(){
  Config prefs(APP_NAME);
  prefs.setGroup("sound");
  prefs.writeEntry("SoundMode", i_currentSettings->SoundMode);
  prefs.writeEntry("sound_enable_login", i_currentSettings->sound_enable_login);
  prefs.writeEntry("sound_enable_logout", i_currentSettings->sound_enable_logout);

  prefs.setGroup("network");
  prefs.writeEntry("login_server_toc", i_currentSettings->login_server_toc );
  prefs.writeEntry("login_server_login", i_currentSettings->login_server_login);
  prefs.writeEntry("login_port_toc", i_currentSettings->login_port_toc);
  prefs.writeEntry("login_port_login", i_currentSettings->login_port_login );
  prefs.writeEntry("login_enable_auto", i_currentSettings->login_enable_auto);
  prefs.writeEntry("login_name", i_currentSettings->login_name );
  prefs.writeEntry("proxy", i_currentSettings->proxy);
  prefs.writeEntry("proxy_server" , i_currentSettings->proxy_server );
  prefs.writeEntry("proxy_port" , i_currentSettings->proxy_port );

  prefs.setGroup("general");
  prefs.writeEntry("main_showIdle", i_currentSettings->main_showIdle);
  prefs.writeEntry("main_browser", i_currentSettings->main_browser);
  prefs.writeEntry("main_showLastOn", i_currentSettings->main_showLastOn);
  prefs.writeEntry("main_noIcons", i_currentSettings->main_noIcons);
  prefs.writeEntry("main_idleLayout", i_currentSettings->main_idleLayout);
  prefs.writeEntry("main_noHeader", i_currentSettings->main_noHeader);
	
  prefs.setGroup("chat");
  prefs.writeEntry("chat_raise_window", i_currentSettings->chatSettings->chat_raise_window);
  prefs.writeEntry("chat_show_time", i_currentSettings->chatSettings->chat_show_time );
  prefs.writeEntry("chat_return_send", i_currentSettings->chatSettings->chat_return_send);
  prefs.writeEntry("chat_ignore_buddyhtml", i_currentSettings->chatSettings->chat_ignore_buddyhtml);
  prefs.writeEntry("chat_show_smilies", i_currentSettings->chatSettings->chat_show_smilies);
  prefs.writeEntry("chat_line_limit", i_currentSettings->chatSettings->chat_line_limit);
  prefs.writeEntry("chat_line_limit_number", i_currentSettings->chatSettings->chat_line_limit_number);

  // Chat Toolbar
  prefs.writeEntry("singleChat_view_text", i_currentSettings->chatToolBarSettings->view_text);
  prefs.writeEntry("singleChat_view_chat", i_currentSettings->chatToolBarSettings->view_chat);
  prefs.writeEntry("singleChat_loc_text", i_currentSettings->chatToolBarSettings->loc_text);
  prefs.writeEntry("singleChat_loc_chat" , i_currentSettings->chatToolBarSettings->loc_chat);
		
  prefs.writeEntry("chat_sound_enable", i_currentSettings->chatSoundSettings->chat_sound);
  prefs.writeEntry("chat_sound_beepInitial", i_currentSettings->chatSoundSettings->chat_sound_beepInitial);
  prefs.writeEntry("chat_sound_beepReceive", i_currentSettings->chatSoundSettings->chat_sound_beepReceive);
  prefs.writeEntry("chat_sound_beepSend", i_currentSettings->chatSoundSettings->chat_sound_beepSend);

  prefs.setGroup("away list");
  prefs.clearGroup();
  QMap <QString,QString>::Iterator it;
  int i = 0;
  for(i = 1, it = i_currentSettings->mapAwayMessages->begin(); it != i_currentSettings->mapAwayMessages->end(); ++it, ++i){
    prefs.writeEntry(QString("away_message_%1").arg(i), it.key());
    prefs.writeEntry(QString("away_message_text%1").arg(i), it.data());
  }	
  prefs.writeEntry("away_message_count", i-1); //i-1 because of the last ++i
}

/******************************************************************************
 * Returns a string (in "config" format) of the passed user's buddy list
 * (in other words, converts the xml buddy list to config format)
 * this is only a helper function for the below function
 ******************************************************************************/
QString setup::buddyListToString(const QString &user){
  if (user.isEmpty() || user == QString("<new user>"))
		return QString("");
	
	TBuddyList buddyList, permitList, denyList;
	int status;

	parseBuddyList(user, &buddyList, &permitList, &denyList, &status);

	return buddyListToString(&buddyList, &permitList, &denyList, status);

}

/******************************************************************************
 * Returns a string (in "config" format) of the passed user's buddy list
 * (in other words, converts the xml buddy list to config format)
 ******************************************************************************/
QString setup::buddyListToString(const TBuddyList *buddyList, const TBuddyList *permitList, const TBuddyList *denyList, int permitStatus){
  QString data = QString("m %1\n").arg(permitStatus);

  int i = -1;
  for (int group =0; group < buddyList->getCountGroup(); group++){
    data += "g " + buddyList->getNameGroup(group) + "\n";
    // int currentGroup = buddyList->getGroup(group);
    i = -1;
    while(++i < buddyList->getCount()){
      if(group == buddyList->getGroup(i))
         data += "b " + buddyList->getName(i) + ":" + buddyList->getAlias(i) + "\n";
    }
  }

  i = -1;
  while(++i < permitList->getCount()) {
    data += "p " + permitList->getName(i) + "\n";
  }
  i = -1;
  while(++i < denyList->getCount()) {
    data += "d " + denyList->getName(i) + "\n";
  }

  return data;
}

/******************************************************************************
 * Saves the config
 ******************************************************************************/
void setup::saveBuddyList(const QString &user, const TBuddyList *buddyList, const TBuddyList *permitList, const TBuddyList *denyList, int permitStatus){
  if(user.isEmpty() || user == QString("<new user>"))
    return;
    
  int buddyCount = 0;
  int group = 0;
  Config prefs(APP_NAME);
  prefs.setGroup( user + " buddy list");
  prefs.writeEntry( "permit status", permitStatus );
  prefs.clearGroup();

  for( group = 0; group < buddyList->getCountGroup(); group++){
    prefs.writeEntry( QString("buddy_group_%1").arg(group + 1), buddyList->getNameGroup(group));

    for(int i = 0; i < buddyList->getCount(); i++){
      if(group == buddyList->getGroup(i)){
        buddyCount++;
        prefs.writeEntry( QString("buddy_%1_name").arg(buddyCount), buddyList->getName(i) );
        prefs.writeEntry( QString("buddy_%1_group").arg(buddyCount), group );
	if( !buddyList->getAlias(i).isEmpty() )
	  prefs.writeEntry( QString("buddy_%1_alias").arg(buddyCount), buddyList->getAlias(i) );
      }
    }
  }
  prefs.writeEntry( "buddy_group_count", group );
  prefs.writeEntry( "buddy_count", buddyCount );
  
  //Permit_Deny
  for(int i = 0; i < permitList->getCount(); i++)
    prefs.writeEntry( QString("permit_%1_name").arg(i), permitList->getName(i) );
  prefs.writeEntry( "permit_group_count", permitList->getCount() );
	
  //Deny
  for(int i = 0; i < denyList->getCount(); i++)
    prefs.writeEntry( QString("deny_%1_name").arg(i), denyList->getName(i) );
  prefs.writeEntry( "deny_group_count", denyList->getCount());
}

#define readNumEntry( a, b ) prefs.readEntry(a, QString("%1").arg(b)).toInt()


/******************************************************************************
 * Reads the buddylist (helper function for below)
 ******************************************************************************/
void setup::parseBuddyList(const QString &user,
   TBuddyList *buddyList,
   TBuddyList *permitList,
   TBuddyList *denyList,
   int *permitStatus){
  if (user.isEmpty() || user == QString("<new user>")){
    buddyList->addGroup("Buddies"); //default group, if there are none
    (*permitStatus) = 1;
    return;
  }
  
  Config prefs(APP_NAME);
  prefs.setGroup( user + " buddy list");
  (*permitStatus) = readNumEntry( "permit status", 1 );

  // Buddylist
  TBuddy buddy;
  int groupCount = readNumEntry( "buddy_group_count", -1);
  for( int i = 0; i < groupCount; i++){
    QString newGroup = prefs.readEntry( QString("buddy_group_%1").arg(i + 1) );
    int groupNumber = buddyList->addGroup(newGroup);
    if (groupNumber == -1)
      groupNumber = buddyList->getNumGroup(newGroup);

    for(int b = 0; b < readNumEntry("buddy_count", -1); b++){
      int buddyGroup = readNumEntry( QString("buddy_%1_group").arg(b+1), -1);
      if( buddyGroup == i){
        buddy.status = TAIM_OFFLINE;
        buddy.group = groupNumber;
        buddy.lastOn = 0;
        buddy.name = prefs.readEntry( QString("buddy_%1_name").arg(b+1) );
        buddy.alias = prefs.readEntry( QString("buddy_%1_alias").arg(b+1), buddy.name );

        // Easter Egg Developer names get put to their real names.
        if( buddy.name.lower() == QString("icefox2") )
          if( buddy.name == buddy.alias)
            buddy.alias = "Ben Meyer";
        if( buddy.name.lower() == QString("remo177") )
          if( buddy.name == buddy.alias)
            buddy.alias = "Dimi Shahbaz";
        if( buddy.name.lower() == QString("jimwinme") )
          if( buddy.name == buddy.alias)
            buddy.alias = "Jim Wilson";
        if( buddy.name.lower() == QString("Dr HP Baldy") )
          if( buddy.name == buddy.alias)
            buddy.alias = "Adam Keys";
        buddyList->add(&buddy);
      }
    }
  }			

  //Permit
  int permitCount = readNumEntry( "permit_group_count", 0);
  for(int i = 0; i < permitCount; i++){
    buddy.name = prefs.readEntry( QString("permit_%1_name").arg(i));
    permitList->add(&buddy);
  }
  
  //Deny
  int denyCount = readNumEntry( "deny_group_count", 0);
  for(int i = 0; i < denyCount; i++){
    buddy.name = prefs.readEntry( QString("deny_%1_name").arg(i));
    denyList->add(&buddy);
  }

  if(groupCount == 0){
    buddyList->addGroup("Buddies"); //default group, if there are none
  }
}

/***************************************************************************
 * Read the settings from the file
 ***************************************************************************/
void setup::read(){	
  int i, r, g, b;
  r = g = b = 0;
  Config prefs(APP_NAME);

  prefs.setGroup("sound");
  i_currentSettings->SoundMode= readNumEntry("SoundMode", 1);
  i_currentSettings->sound_enable_login	= prefs.readBoolEntry("sound_enable_login", true);
  i_currentSettings->sound_enable_logout= prefs.readBoolEntry("sound_enable_logout", false );

  prefs.setGroup("network");
  i_currentSettings->login_server_toc = prefs.readEntry("login_server_toc", "toc.oscar.aol.com");
  i_currentSettings->login_server_login= prefs.readEntry("login_server_login", "login.oscar.aol.com");
  i_currentSettings->login_port_toc= prefs.readEntry("login_port_toc", "9898");
  i_currentSettings->login_port_login= prefs.readEntry("login_port_login", "5190");
  i_currentSettings->login_enable_auto= prefs.readBoolEntry("login_enable_auto", false);
  i_currentSettings->login_name = prefs.readEntry("login_name", "<new user>");
  i_currentSettings->proxy = prefs.readBoolEntry("proxy", false);
  i_currentSettings->proxy_server = prefs.readEntry("proxy_server", "");
  i_currentSettings->proxy_port = prefs.readEntry("proxy_port", "");

  prefs.setGroup("general");
  i_currentSettings->main_showIdle= prefs.readBoolEntry("main_showIdle", true);
  i_currentSettings->main_browser = prefs.readEntry("main_browser", "konqueror");
  i_currentSettings->main_showLastOn= prefs.readBoolEntry("main_showLastOn", true);
  i_currentSettings->main_noIcons= prefs.readBoolEntry("main_noIcons", false);
  i_currentSettings->main_idleLayout= readNumEntry("main_idleLayout", 0);
  i_currentSettings->main_noHeader= prefs.readBoolEntry("main_noHeader", false); 

  prefs.setGroup("chat");
  i_currentSettings->chatSettings->chat_raise_window= prefs.readBoolEntry("chat_raise_window", true);
  i_currentSettings->chatSettings->chat_show_time = prefs.readBoolEntry("chat_show_time", true);
  i_currentSettings->chatSettings->chat_return_send= prefs.readBoolEntry("chat_return_send", true);
  i_currentSettings->chatSettings->chat_ignore_buddyhtml= prefs.readBoolEntry("chat_ignore_buddyhtml", false);
  i_currentSettings->chatSettings->chat_show_smilies= prefs.readBoolEntry("chat_show_smilies", true);
  i_currentSettings->chatSettings->chat_line_limit= prefs.readBoolEntry("chat_line_limit", false);
  i_currentSettings->chatSettings->chat_line_limit_number= readNumEntry("chat_line_limit_number",200 );

  i_currentSettings->chatToolBarSettings->view_text= prefs.readBoolEntry("singleChat_view_text", true);
  i_currentSettings->chatToolBarSettings->view_chat= prefs.readBoolEntry("singleChat_view_chat", true);
  i_currentSettings->chatToolBarSettings->loc_text= readNumEntry("singleChat_loc_text", 0);
  i_currentSettings->chatToolBarSettings->loc_chat= readNumEntry("singleChat_loc_chat", 0);

   i_currentSettings->chatSoundSettings->chat_sound= prefs.readBoolEntry("chat_sound_enable", true);
  i_currentSettings->chatSoundSettings->chat_sound_beepInitial= prefs.readBoolEntry("chat_sound_beepInitial", true);
  i_currentSettings->chatSoundSettings->chat_sound_beepReceive= prefs.readBoolEntry("chat_sound_beepReceive", true);
  i_currentSettings->chatSoundSettings->chat_sound_beepSend= prefs.readBoolEntry("chat_sound_beepSend", true);

  prefs.setGroup("away list");
  i_currentSettings->mapAwayMessages->clear();
  int totalAwayMessages = readNumEntry("away_message_count", -1);
  for(i = 1; i <= totalAwayMessages; i++){
    i_currentSettings->mapAwayMessages->insert(
    prefs.readEntry(QString("away_message_%1").arg(i)),
    prefs.readEntry(QString("away_message_text%1").arg(i)) );
  }

  if(totalAwayMessages == 0)
    i_currentSettings->mapAwayMessages->insert("Away", "I am away right now, leave a message.");

}

void setup::savePassword(const QString &user, const QString &password){
  if(user.isEmpty() || (user == QString("<new user>")))
    return;
  
  Config prefs(APP_NAME);
  prefs.setGroup("savedUsers");
  QStringList users = prefs.readListEntry("users",'/');
  if( !isRegisteredUser(user) ){
    users += user;
    prefs.writeEntry ( "users", users, '/' );
  }

  if( password.isEmpty())
    return;

  prefs.setGroup("password");
  prefs.writeEntry(user, password);
}

QString setup::readPassword(const QString &user){
  Config prefs(APP_NAME);
  prefs.setGroup("password");
  return prefs.readEntry(user);	
}

void setup::removePassword(const QString &user){
  if(user.isEmpty() || (user == QString("<new user>")))
    return;

  Config prefs(APP_NAME);
  prefs.setGroup("savedUsers");
  QStringList users = prefs.readListEntry("users",'/');
  if( !isRegisteredUser(user)) {
    users += user;
    prefs.writeEntry ( "users", users, '/' );
  }

  prefs.setGroup("password");
  return prefs.writeEntry(user, "");	
}

void setup::saveProfile(const QString &user, const QString &profile){
  if(user.isEmpty() || user == QString("<new user>"))
    return;

  Config prefs(APP_NAME);
  prefs.setGroup("profile");
  prefs.writeEntry(user, profile);	
}
	
QString setup::readProfile(const QString &user){
  Config prefs(APP_NAME);
  prefs.setGroup("profile");
  return prefs.readEntry(user, DEFAULT_PROFILE);	
}

/**
 * Reset the default settings.
 */
void setup::resetDefaults(){
  Config prefs(APP_NAME);
  prefs.setGroup("away");
  prefs.clearGroup();
  prefs.setGroup("chat");
  prefs.clearGroup();
  prefs.setGroup("general");
  prefs.clearGroup();
  prefs.setGroup("network");
  prefs.clearGroup();
  prefs.setGroup("sound");
  prefs.clearGroup();
  
  read();
}


// setup.cpp

