/***************************************************************************
                                  setup.h
                             -------------------
    begin                : Thu Feb 17 2000
    copyright            : (C) 2000 by Benjamin Meyer
    email                : benjamin@csh.rit.edu
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SETUP_H
#define SETUP_H

#include <qstring.h>
#include <qtextstream.h>
#include <qfile.h>
#include <qdict.h>
#include <qmessagebox.h>
#include <stdlib.h>
#include <qmap.h>

#include "tbuddylist.h"

struct chat_Settings {		
  bool chat_raise_window;         // Done
  bool chat_show_time;            // Done
  bool chat_return_send;	  			// Done
  bool chat_ignore_buddyhtml;     // Done
  bool chat_line_limit;           // Done
  bool chat_show_smilies;         // Done
  int chat_line_limit_number;     // Done
};

struct chat_Sound_Settings{
  bool chat_sound;                // Done
  bool chat_sound_beepInitial;     // Done
  bool chat_sound_beepReceive;    // Done
  bool chat_sound_beepSend;       // Done
};

struct chat_ToolBar_Settings{
  bool view_text;                 // Done
  bool view_chat;                 // Done

  // 0 top
  // 1 right
  // 2 bottom
  // 3 left                       // Done
  int loc_text;                   // Done
  int loc_chat;                   // Done
};

struct settings_t {		
  QString config;
    
  // Sound
  int SoundMode;
  bool sound_enable_login;
  bool sound_enable_logout;
 	
  // Network
  QString login_server_toc;
  QString login_server_login;
  QString login_port_toc;
  QString login_port_login;
  bool login_enable_save;					// In Use
  bool login_enable_auto;					// In Use
  QString login_name;					// In Use
  QString login_password;					// In Use
  bool proxy;
  QString proxy_server;
  QString proxy_port;
  	
  // Permit Deny
 
  // Away
  QMap <QString, QString> *mapAwayMessages;
		
  // General
  bool main_showIdle;
  bool main_showLastOn;
  QString main_browser;
  bool main_noIcons;
  int main_idleLayout;
  bool main_noHeader;

  // Chat Settings
  chat_Settings *chatSettings;
  chat_Sound_Settings *chatSoundSettings;
  chat_ToolBar_Settings *chatToolBarSettings;

  // Internal Settings (not to be saved)
  QString Version;
  QString ReleaseDate;
};

/**
  *@author Benjamin Meyer
  */

class setup {
	
  public:
    setup();
    ~setup();
  
  static setup *instance();
  static settings_t *settings();
				
    void save();
    void read();
    void resetDefaults();

    QString readPassword(const QString &user);
    void savePassword(const QString &user, const QString &password);
    void removePassword(const QString &user);
    QStringList registeredUsers(void);
    bool isRegisteredUser(const QString &user);
    QString readProfile(const QString &user);
    void saveProfile(const QString &user, const QString &profile);
    void saveBuddyList(const QString &user, 
	    const TBuddyList *buddyList, 
	    const TBuddyList *permitList, 
	    const TBuddyList *denyList, int permitStatus);
    void parseBuddyList(const QString &user, 
	    TBuddyList *buddyList, 
	    TBuddyList *permitList, 
	    TBuddyList *denyList, int *permitStatus);
    QString buddyListToString(const QString &user);
    QString buddyListToString(const TBuddyList *buddyList, 
	    const TBuddyList *permitList, 
	    const TBuddyList *denyList, int permitStatus);
  private:
    settings_t *i_currentSettings;
};

#endif

// setup.h

